

// Interfacing functions are placed in this place


function [sim] = ld_savefile(sim, events, fname, source, vlen) // PARSEDOCU_BLOCK
//
// %PURPOSE: Quick and easy dumping of signals to files
// 
// fname - string of the filename
// source *+ - source signal
// vlen - vector size of signal
// 

  [inp] = libdyn_extrakt_obj( source ); // compatibility

  autostart = 1;
  maxlen = 0
  fname = ascii(fname);

  btype = 130;

  [sim,blk] = libdyn_new_block(sim, events, btype, [maxlen, autostart, vlen, length(fname), fname(:)'], [],  ...
					insizes=[ vlen ], outsizes=[], ...
					intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[]  );
  
  [sim,save_] = libdyn_conn_equation(sim, blk, list(source) );
endfunction


function [sim, out] = ld_switch2to1(sim, events, cntrl, in1, in2) // PARSEDOCU_BLOCK
//
// %PURPOSE: A 2 to 1 switching Block
//
// cntr * - control input
// in1 *
// in2 *
// out * - output
//
// if cntrl > 0 : out = in1
// if cntrl < 0 : out = in2
//
  btype = 60001;
  [sim,blk] = libdyn_new_block(sim, events, btype, [], [], ...
                   insizes=[1, 1, 1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                   outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(cntrl, in1, in2) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction





function [sim, outlist] = ld_demux(sim, events, vecsize, invec) // PARSEDOCU_BLOCK
//
// %PURPOSE: Demultiplexer
//
// invec * - input vector signal to be split up
// outlist *LIST - list() of output signals
//
//
// Splits the input vector signal "invec" of size "vecsize" up into 
//
// outlist(1)
// outlist(2)
//  ....
//    
  btype = 60001 + 1;	
  ipar = [vecsize, 0]; rpar = [];
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[vecsize], outsizes=[ones(vecsize,1)], ...
                       intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT*ones(vecsize,1)]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(invec) );

  // connect each outport
  outlist = list();
  for i = 1:vecsize
    [sim,out] = libdyn_new_oport_hint(sim, blk, i-1);   // ith port
    outlist(i) = out;
  end
endfunction



function [sim, out] = ld_mux(sim, events, vecsize, inlist) // PARSEDOCU_BLOCK
//    
// %PURPOSE: Multiplexer
//
// inlist *LIST - list() of input signals of size 1
// out *+ - output vector signal
// 
// 
// combines inlist(1), inlist(2), ...    
// to a vector signal "out" of size "vecsize", whereby each inlist(i) is of size 1
//    
  btype = 60001 + 2;	
  ipar = [vecsize; 0]; rpar = [];

  if (length(inlist) ~= vecsize) then
    printf("Incorect number of input ports to ld_mux. %d != %d\n", length(inlist), vecsize );
    error(".");
  end

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[ones(1,vecsize)], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT*ones(1,vecsize) ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, inlist );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_hysteresis(sim, events, in, switch_on_level, switch_off_level, initial_state, onout, offout) // PARSEDOCU_BLOCK
// %PURPOSE: hysteresis block
//
// in * - input
// out * -output
// 
// switches out between onout and offout
// initial state is either -1 (off) or 1 (on)
//
//

  if (switch_off_level > switch_on_level) then
    error("ld_hysteresis: setting switch_off_level > switch_on_level makes no sense\n");
  end

  btype = 60001 + 3;
  [sim,blk] = libdyn_new_block(sim, events, btype, [initial_state], [ switch_on_level, switch_off_level, onout, offout] , ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_modcounter(sim, events, in, initial_count, mod) // PARSEDOCU_BLOCK
// %PURPOSE: Modulo Counter - Block
//
// in * - input
// out * -output
// 
// A counter that increases its value for each timestep for which in > 0 is true.
// if the counter value >= mod then it is reset to counter = initial_count
//
//

  if (mod < 0) then
    error("ld_modcounter: mod is less than zero\n");
  end

  btype = 60001 + 4;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ initial_count, mod ], [  ],  ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]   );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_jumper(sim, events, in, steps) // PARSEDOCU_BLOCK
// %PURPOSE: jumper - block
//
// out *+ - vector of size steps
// in * - switching input
//
// The vector out always contains one "1", the rest is zero.
// The "1" moves to the right if in > 0. If the end is reached
// it "1" flips back to the left side
// 
// 

  if (steps <= 0) then
    error("ld_jumper: steps must be greater than zero\n");
  end

  btype = 60001 + 5;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ steps ], [  ], ...
                   insizes=[1], outsizes=[ steps ], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_memory(sim, events, in, rememberin, initial_state) // PARSEDOCU_BLOCK
// %PURPOSE: memory - block
//
// in * - input
// rememberin * - 
// out * - output
// 
// If rememberin > 0 then
//   remember in, which is then feed to the output out until it is overwritten by a new value
//
// initial output out = initial_state
// 

  memsize = length(initial_state);

  btype = 60001 + 6;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ memsize  ], [ initial_state ],  ...
                   insizes=[memsize, 1], outsizes=[memsize], ...
                   intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]   );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in, rememberin) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_abs(sim, events, in) // PARSEDOCU_BLOCK
// %PURPOSE: abs - block
//
// in * - input
// out * - output
// 
// out = abs(in)
// 

  btype = 60001 + 7;
  [sim,blk] = libdyn_new_block(sim, events, btype, [  ], [  ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_extract_element(sim, events, invec, pointer, vecsize ) // PARSEDOCU_BLOCK
  //
  // %PURPOSE: Extract one element of a vector
  //
  // invec *+ - the input vector signal
  // pointer * - the index signal
  // vecsize - length of input vector
  // 
  // out = invec[pointer], the first element is at pointer = 1
  //

  btype = 60001 + 8;	
  ipar = [ vecsize, ORTD.DATATYPE_FLOAT ]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[vecsize, 1], outsizes=[1], ...
                       intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

//   [sim,blk] = libdyn_new_blk_generic(sim, events, btype, ipar, rpar               );


  [sim,blk] = libdyn_conn_equation(sim, blk, list(invec, pointer) );

  // connect each outport
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // ith port
endfunction


function [sim, out] = ld_constvec(sim, events, vec) // PARSEDOCU_BLOCK
// 
// %PURPOSE: a constant vector
// 
// vec *+ - the vector
// 
  btype = 60001 + 9;	
  ipar = [length(vec); 0]; rpar = [vec];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                   insizes=[1], outsizes=[ length(vec) ], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );
 
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_counter(sim, events, count, reset, resetto, initial) // PARSEDOCU_BLOCK
// 
// %PURPOSE: A resetable counter block
//
// count * - signal
// reset * - signal
// resetto * - signal
// initial - constant
// out * - output
// 
// increases out by count (out = out + count)
// 
// if reset > 0.5 then
//   out = resetto
//
// initially out is set to initial
// 
// 

  btype = 60001 + 10;
  ipar = [  ]; rpar = [ initial ];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[1,1,1], outsizes=[1], ...
                       intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                       outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list( count, reset, resetto ) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_shift_register(sim, events, in, len) // FIXME TODO
// %PURPOSE: A shift register with access to the stored values
//
// in *
// out *+(len)
//    
// 
// 
// 
  btype = 60001 + 11;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[ 1 ], outsizes=[len], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_lookup(sim, events, u, lower_b, upper_b, table, interpolation) // PARSEDOCU_BLOCK
// %PURPOSE: Lookup table - block
//
// in * - input
// out * - output
// 
// 
// lower_b - smallest value of the input signal to map to the table
// upper_b - biggest value of the input signal to map to the table
// table - the table (Scilab vector)
// 
// Mapping is done in a linear way:
//   out = table( (in - lowerin) / (upperin - lowerin) )
// 
// interpolation = 0 : no interpolation
// interpolation = 1 : linear interpolation
// 
// 

  btype = 60001 + 12;
  [sim,blk] = libdyn_new_block(sim, events, btype, [length(table), interpolation ], [ lower_b, upper_b, table(:)' ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(u) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_not(sim, events, in) // PARSEDOCU_BLOCK
// %PURPOSE: logic negation - block
//
// in * - input
// out * - output
// 
// out = 0, if in > 0.5  OR  out = 1, if in < 0.5
// 

  btype = 60001 + 13;
  [sim,blk] = libdyn_new_block(sim, events, btype, [  ], [  ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_or(sim, events, inlist) // PARSEDOCU_BLOCK
// %PURPOSE: logic or - block
//
// in *LIST - list() of inputs (for now the exactly two inputs are possible)
// out * - output
// 
// 
// 
  Nin=length(inlist);

  if (Nin ~= 2) then
    error("invalid number of inputs");
  end

  insizes=ones(1, Nin);
  intypes=ones(1, Nin) * ORTD.DATATYPE_FLOAT;

  btype = 60001 + 14;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[  ], rpar=[   ], ...
                   insizes, outsizes=[1], ...
                   intypes, outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list( inlist(1), inlist(2) ) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_iszero(sim, events, in, eps) // PARSEDOCU_BLOCK
//
// %PURPOSE: check if input is near zero
//
// in * - input
// out * - output
// 
// out = 1, if in between -eps and eps, othwewise out = 0
// 

  btype = 60001 + 15;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[  ], rpar=[ eps ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_limitedcounter(sim, events, count, reset, resetto, initial, lower_b, upper_b) // PARSEDOCU_BLOCK
// 
// %PURPOSE: A resetable, limited counter block
//
// count * - signal
// reset * - signal
// resetto * - signal
// initial - constant
// out * - output
// 
// increases out by count (out = out + count), but count is always between lower_b and upper_b
// 
// if reset > 0.5 then
//   out = resetto
//
// initially out is set to initial
// 
// 

  if (lower_b > upper_b) then
    error("lower_b is greater than upper_b");
  end

  btype = 60001 + 16;
  ipar = [  ]; rpar = [ initial, lower_b, upper_b ];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[1,1,1], outsizes=[1], ...
                       intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                       outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list( count, reset, resetto ) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_memorydel(sim, events, in, rememberin, initial_state) // PARSEDOCU_BLOCK
// %PURPOSE: delayed memory - block
//
// in * - input
// rememberin * - 
// out * - output
// 
// If rememberin > 0 then
//   remember in, which is then feed to the output "out" in the next time step until it is overwritten by a new value
//
// initial output out = initial_state
// 

  memsize = length(initial_state);

  btype = 60001 + 17;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ memsize  ], [ initial_state ],  ...
                   insizes=[memsize, 1], outsizes=[memsize], ...
                   intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]   );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in, rememberin) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_steps(sim, events, activation_simsteps, values) // PARSEDOCU_BLOCK
//
// %PURPOSE: steps
//
// out * - output
// 
// 

  if (length(activation_simsteps) ~= length(values)-1) then
    error("length(activation_simsteps) != length(values)-1");
  end

  btype = 60001 + 18;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ length(values), activation_simsteps ], rpar=[ values ], ...
                   insizes=[], outsizes=[1], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );


  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_cond_overwrite(sim, events, in, condition, setto) // PARSEDOCU_BLOCK
//
// %PURPOSE: conditional overwrite of the input signal's value
//
// out * - output
// 
// out = in, if condition < 0.5
// out = setto, otherwise
// 


  btype = 60001 + 19;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ ], rpar=[ setto ], ...
                   insizes=[1,1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(condition, in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_ramp(sim, events, in_from, in_to, start, reset, ramp_duration) // NOT FINISHED
//
// %PURPOSE: Online configurable ramp block
//
// out * - output (from 0 to 1)
// start * - if > 0.5 the ramp starts
// reset * - if > 0.5 the blocks states are reset
// increase * - constant by which the output is increased for each time step
// 
// 

  btype = 60001 + 20;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ ], rpar=[  ], ...
                   insizes=[1,1,1], outsizes=[1], ...
                   intypes=ORTD.DATATYPE_FLOAT*[1,1,1], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(  start, reset, increase ) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_and(sim, events, inlist) // PARSEDOCU_BLOCK
// %PURPOSE: logic and - block
//
// in *LIST - list() of inputs (for now the exactly two inputs are possible)
// out * - output
// 
// 

  Nin=length(inlist);

  if (Nin ~= 2) then
    error("invalid number of inputs");
  end

  insizes=ones(1, Nin);
  intypes=ones(1, Nin) * ORTD.DATATYPE_FLOAT;

  btype = 60001 + 21;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[  ], rpar=[   ], ...
                   insizes, outsizes=[1], ...
                   intypes, outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list( inlist(1), inlist(2) ) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_initimpuls(sim, events) // PARSEDOCU_BLOCK
//
// %PURPOSE: initial impuls
//
// out * - output
// 
// 


  btype = 60001 + 22;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ ], rpar=[ ], ...
                   insizes=[], outsizes=[1], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );


  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim] = ld_printfstderr(sim, events, in, str, insize) // PARSEDOCU_BLOCK
//
// %PURPOSE: Print data to stderr (the console)
//
// in *+(insize) - vectorial input signal
//
// str is a string that is printed followed by the signal vector in
// of size insize
//
  //[sim,blk] = libdyn_new_printf(sim, events, str, insize);
  btype = 60001 + 23;;
  str = ascii(str);
//   [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [insize, length(str), str(:)'], []);

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ insize, length(str), str(:)' ], rpar=[ ], ...
                   insizes=[ insize ], outsizes=[], ...
                   intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
endfunction

function [sim, out] = ld_delay(sim, events, u, N) // PARSEDOCU_BLOCK
// %PURPOSE: delay - block
//
// in * - input
// out * - output
// 
// delay in by N steps
// 
// 

  if length(N) ~= 1 then
    error("N is not a scalar\n");
  end

  if (N < 1) then
    error("invalid delay");
  end

  btype = 60001 + 24;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ N ], [ ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(u) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction



function [sim, out] = ld_steps2(sim, events, activation_simsteps, values) // PARSEDOCU_BLOCK
//
// %PURPOSE: steps, counter is increased on event, which is different to ld_steps
// (EXPERIMENTAL because not tested for now)
//
// out * - output
// 
// 

  if (length(activation_simsteps) ~= length(values)-1) then
    error("length(activation_simsteps) != length(values)-1");
  end

  btype = 60001 + 25;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ length(values), activation_simsteps ], rpar=[ values ], ...
                   insizes=[], outsizes=[1], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );


  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim,out] = ld_getsign(sim, events, in) // PARSEDOCU_BLOCK
//
// %PURPOSE: return the sign of the input sigal
// either 1 or -1
//

  btype = 60001 + 26;
  [sim,blk] = libdyn_new_block(sim, events, btype, [  ], [  ], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_insert_element(sim, events, in, pointer, vecsize ) // PARSEDOCU_BLOCK
  //
  // %PURPOSE: Insert one element in a vector
  //
  // in *+ - the input element signal
  // pointer * - the index signal
  // vecsize - length of output vector
  // 
  // out[pointer] = in, the first element is at pointer = 1
  //

  btype = 60001 + 27;	
  ipar = [ vecsize, ORTD.DATATYPE_FLOAT ]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[1, 1], outsizes=[vecsize], ...
                       intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in, pointer) );

  // connect each outport
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // ith port
endfunction

function [sim] = ld_FlagProbe(sim, events, in, str, insize) // PARSEDOCU_BLOCK
//
// %PURPOSE: Print data and Flags (calc output, update states, reset states) to stderr (the console)
//
// in *+(insize) - vectorial input signal
//
// str is a string that is printed followed by the signal vector in
// of size insize
//
  //[sim,blk] = libdyn_new_printf(sim, events, str, insize);
  btype = 60001 + 28;
  str = ascii(str);
//   [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [insize, length(str), str(:)'], []);

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ insize, length(str), str(:)' ], rpar=[ ], ...
                   insizes=[ insize ], outsizes=[], ...
                   intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
endfunction


















// 
//  Vector functions
// 

function [sim, out] = ld_vector_delay(sim, events, in, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: delay - block
//
// in * - input
// out * - output
// vecsize - size of vector in*
// 
// delay the hole vector in by one step
// 
// 


  btype = 60001 + 65;
  [sim,blk] = libdyn_new_block(sim, events, btype, [ vecsize ], [ ], ...
                   insizes=[vecsize], outsizes=[vecsize], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_vector_diff(sim, events, in, vecsize) // PARSEDOCU_BLOCK
//    
// %PURPOSE: Vector differentiation with respect to the index
// 
// in *+(vecsize) - vector signal of size "vecsize"
// out *+(vecsize-1) - vector signal of size "vecsize-1"
//
// Equivalent to Scilab 'diff' function
//    
  btype = 60001 + 50;	
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize], outsizes=[vecsize-1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, index] = ld_vector_findthr(sim, events, in, thr, greater, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: find value that is grater as an constant
//
// in *+(vecsize) - input
// thr * - threshold signal
// index * - output
// 
// find values greater than threshold "thr" in vector signal "in", when greater > 0
// find values less than threshold "thr" in vector signal "in", when greater =< 0
// 
//
//    
  btype = 60001 + 51;	
  ipar = [vecsize; greater]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in, thr ) );

  [sim,index] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_abs(sim, events, in, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: Vector abs()
//
// in *+(vecsize) - input
// out *+(vecsize) - output
// 
//    
  btype = 60001 + 52;	
  ipar = [vecsize; 0]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_gain(sim, events, in, gain, vecsize) // PARSEDOCU_BLOCK   
// %PURPOSE: Vector gain
//
// in *+(vecsize) - input
// out *+(vecsize) - output
//    
  btype = 60001 + 53;	
  ipar = [vecsize]; rpar = [gain];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_extract(sim, events, in, from, window_len, vecsize) // PARSEDOCU_BLOCK
//    
// %PURPOSE: Extract "in" from to from+window_len
// 
//  in *+(vecsize) - vector signal
//  from * - index signal
//  out *+(window_len) - output signal
//
//    
  btype = 60001 + 54;	
  ipar = [vecsize; window_len]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1], outsizes=[window_len], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
//   disp('new vextr\n');
//   disp( [vecsize, 1] );


  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in, from ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, index, value] = ld_vector_minmax(sim, events, in, findmax, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: Min / Max of a vector (finds the first appearance of the minimum/maximum)
//
// Function is buggy somehow. Find maximum should work. Minimum perhaps not!
//
// in *+(vecsize)
// findmax greater than 0 means "find the maximum"
// index * - the index starting at 1, where the max / min was found
// value * - min/max value
//    
  btype = 60001 + 55;	
  ipar = [vecsize; findmax]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize], outsizes=[1, 1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,index] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
  [sim,value] = libdyn_new_oport_hint(sim, blk, 1);   // 1th port
endfunction

function [sim, out, num] = ld_vector_glue(sim, events, in1, fromindex1, toindex1, in2, fromindex2, toindex2, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: Extract parts from two input vectors and glue them together to receive one vector.
//
// Output starting with in1 from fromindex1 until toindex1, continuing with in2 from fromindex2 until toindex2.
// Function is under developement!
//
// in1 *+(vecsize)
// in2 *+(vecsize)
// fromindex1 * - first index considered from in1
// toindex1 * - last index considered from in1
// fromindex2 * - first index considered from in2
// toindex2 * - last index considered from in2
// vecsize - size of each input vector. Vectors need to have equal size!
// out * - as explained above. Size of output is (2*vecsize).
// num * - number of values that have been glued together.
// 
//    
  btype = 60001 + 64;
  outsize = 2*vecsize;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1, 1, vecsize, 1, 1], outsizes=[outsize, 1], ...
                                     intypes=[ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT], ...
                                     outtypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT]);
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in1,fromindex1,toindex1,in2,fromindex2,toindex2 ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
  [sim,num] = libdyn_new_oport_hint(sim, blk, 1);   // 1th port
endfunction



function [sim, out] = ld_vector_addscalar(sim, events, in, add, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: add "add" to the vector
// 
//  add * - signal
//  in *+(vecsize) - vector signal
//    
  btype = 60001 + 56;	
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT  ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in, add ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_sum(sim, events, in, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: sum over "in"
//
// in *+(vecsize)
// out *
//    
  btype = 60001 + 57;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize ], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_addsum(sim, events, in1, in2, vecsize) // FIXME TODO 
// %PURPOSE: multiplicate two vectors and calc the sum of the result ( sum( in1 .* in2) )
//
// in1 *+(vecsize)
// in2 *+(vecsize)
// out *
//    
  btype = 60001 + 58;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize,vecsize ], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT,ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in1, in2 ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_abssum(sim, events, in, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: sum over element wise abs() of "in"
//
// in *+(vecsize)
// out *
//    
  btype = 60001 + 59;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize ], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_sqsum(sim, events, in, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: sum over element wise ()^2 of "in"
//
// in *+(vecsize)
// out *
//    
  btype = 60001 + 60;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize ], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_extractandsum(sim, events, in, from, window_len, vecsize) // PARSEDOCU_BLOCK
//    
// %PURPOSE: Extract "in" from "from"-index to "to"-index and sum up (untested) EXPERIMENTAL FOR NOW
// 
//  in *+(vecsize) - vector signal
//  from * - index signal
//  to * - index signal
//
//    
  btype = 60001 + 61;
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1, 1], outsizes=[1], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );

  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in, from, to ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_simplecovar(sim, events, in, shape, vecsize) // PARSEDOCU_BLOCK
//    
// %PURPOSE: stupid covariance function
// 
//  in *+(vecsize) - vector signal
//  shape - vector to compare the input with
//  out *+(length(shape)) - output
//
//    
  btype = 60001 + 62;
  ipar = [vecsize, length(shape) ]; rpar = [ shape ];

  if vecsize<length(shape) then
    error("vecsize<length(shape) !");
  end
  
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize  ], outsizes=[ vecsize-length(shape)+1 ], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );

  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, out] = ld_vector_mute(sim, events, in, from, len, setto, vecsize) // PARSEDOCU_BLOCK
// %PURPOSE: mute a vector from and to a spacified index
// 
//  in *+(vecsize) - vector signal
//  from * - signal (index counting starts at )
//  len * - signal (length of the window to mute)
//  setto * - signal
//
//

  btype = 60001 + 63;	
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize, 1, 1, 1, 1], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT  ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in, from, len, setto ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction



// 
// 
// Special blocks
// 


// obsolete
function [sim,bid] = libdyn_new_interface(sim, events, len)
  btype = 4000;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [len], []);
endfunction


function [sim, out] = ld_interface(sim, events, in, vecsize) // PARSEDOCU_BLOCK   
// Interfacing block
//
// in *+(vecsize) - input
// out *+(vecsize) - output
//    
  btype = 60001 + 1000;	
  ipar = [vecsize]; rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[vecsize], outsizes=[vecsize], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction





//
// Macros
//
//


function [sim,y] = ld_add_ofs(sim, events, u, ofs) // PARSEDOCU_BLOCK
//
// %PURPOSE: Add a constant "ofs" to the signal u; y = u + const(ofs)
//
  [sim,ofs_const] = libdyn_new_blk_const(sim, events, ofs);
  
  [sim,y] = ld_sum(sim, events, list(u,0, ofs_const,0), 1,1);
endfunction


function [sim, y] = ld_mute( sim, ev, u, cntrl, mutewhengreaterzero ) // PARSEDOCU_BLOCK
//    
//    %PURPOSE: Mute a signal based on cntrl-signal
//
//    ev - event
//    u * - input
//    y * - output
//    mutewhengreaterzero - boolean parameter (%T, %F)
//    
//    if mutewhengreaterzero == %T then
//
//      y = 0 for cntrl > 0
//      y = u for cntrl < 0
//
//    else
//
//      y = 0 for cntrl < 0
//      y = u for cntrl > 0
//
//
//
    
    [sim, zero] = ld_const(sim, ev, 0);
    
    if (mutewhengreaterzero == %T) then
      [sim,y] = ld_switch2to1(sim, ev, cntrl, zero, u);
    else
      [sim,y] = ld_switch2to1(sim, ev, cntrl, u, zero);
    end
endfunction



function [sim, y] = ld_limited_integrator(sim, ev, u, min__, max__, Ta) // PARSEDOCU_BLOCK
// %PURPOSE: Implements a time discrete integrator with saturation of the output between min__ and max__
// 
// u * - input
// y * - output
// 
// y(k+1) = sat( y(k) + Ta*u , min__, max__ )
    [sim, u__] = ld_gain(sim, ev, u, Ta);
    
    [sim,z_fb] = libdyn_new_feedback(sim);
    
	[sim, sum_] = ld_sum(sim, ev, list(u__, z_fb), 1, 1);
	[sim, tmp] = ld_ztf(sim, ev, sum_, 1/z);
	[sim, y] = ld_sat(sim, ev, tmp, min__, max__);
    
    [sim] = libdyn_close_loop(sim, y, z_fb);    
endfunction

function [sim, y] = ld_limited_integrator2(sim, ev, u, min__, max__, Ta) // PARSEDOCU_BLOCK
// %PURPOSE: Implements a time discrete integrator with saturation of the output between min__ and max__
// compared to ld_limited_integrator there is no delay: Ta z / (z-1)
//
// u * - input
// y * - output
// 
// y(k+1) = sat( y(k) + Ta*u , min__, max__ )


    [sim, u__] = ld_gain(sim, ev, u, Ta);
    
    [sim,z_fb] = libdyn_new_feedback(sim);
    
	[sim, sum_] = ld_sum(sim, ev, list(u__, z_fb), 1, 1);
	[sim, y] = ld_sat(sim, ev, sum_, min__, max__);


	[sim, y_] = ld_ztf(sim, ev, y, 1/z);
    
    [sim] = libdyn_close_loop(sim, y_, z_fb);    

endfunction

function [sim, y] = ld_limited_integrator3(sim, ev, u, min__, max__, Ta) // PARSEDOCU_BLOCK
// %PURPOSE: Implements a time discrete integrator (trapeziodal rule) with saturation of the output between min__ and max__
//
// u * - input
// y * - output
// 
// y(k+1) = sat(  Ta/2 ( u(k)+u(k+1) ) + y(k), min__, max__ )

    ukp1 = u;
    [sim, uk] = ld_ztf(sim, ev, ukp1, 1/z);
    [sim, u__] = ld_add(sim, ev, list(uk, ukp1), [Ta/2, Ta/2] );     
//     [sim, u__] = ld_gain(sim, ev, tmp, Ta/2);
    
    [sim,z_fb] = libdyn_new_feedback(sim);
    
	[sim, sum_] = ld_sum(sim, ev, list(u__, z_fb), 1, 1);
	[sim, y] = ld_sat(sim, ev, sum_, min__, max__);

	[sim, y_] = ld_ztf(sim, ev, y, 1/z);
    
    [sim] = libdyn_close_loop(sim, y_, z_fb);    
endfunction

function [sim, u] = ld_lin_awup_controller(sim, ev, r, y, Ta, tfR, min__, max__) // PARSEDOCU_BLOCK
// %PURPOSE: linear controller with anti reset windup implemented by bounding the integral state:
// e = r-y
// u = ld_limited_integrator( e, min__, max__ ) + tfR*e
    [sim, e] = ld_sum(sim, ev, list(r, y), 1, -1);
    
    [sim,u1] = ld_limited_integrator(sim, ev, e, min__, max__, Ta);
    [sim,u2] = ld_ztf(sim, ev, e, tfR);
    
    [sim,u] = ld_sum(sim, ev, list(u1,u2), 1,1);
endfunction


function [sim] = ld_print_angle(sim, ev, alpha, text) // PARSEDOCU_BLOCK
// %PURPOSE: Convert an angle in rad to degree and print to console
// 
// alpha * - angle signal
// text - string
// 
    [sim, alpha_deg] = ld_gain(sim, ev, alpha, 1/%pi*180);
    [sim] = ld_printf(sim, ev, alpha_deg, text, 1);
endfunction

function [sim, pwm] = ld_pwm(sim, ev, plen, u) // PARSEDOCU_BLOCK
// 
// %PURPOSE: PWM generator
// 
// plen - period length
// u * - modulation signal; Values are between 0 and 1.
// pwm * - pwm output
//

    [sim,u] = ld_gain(sim, ev, u, plen);
    
    [sim,one] = ld_const(sim, ev, 1);
    
    [sim,modcount] = ld_modcounter(sim, ev, in=one, initial_count=0, mod=plen);
    
    [sim, test] = ld_add(sim, ev, list(modcount, u), [-1,1] );
    [sim,pwm] = ld_compare_01(sim, ev, test,  thr=0);
endfunction

function [sim, outvec, Nvecplay] = ld_vector_play(sim, ev, A, special) // PARSEDOCU_BLOCK
// 
// %PURPOSE: Play a vectorial signal
// 
// A - matrix containing the vectors to play
// outvec*+(length(A(:,1))) - 
// Nvecplay = length of output signal vector
//
// outputs A(:,i), where i is increasing within each time step
// special = [ "repeate" ]
//

  [Nvecplay,Nsamples] = size(A); // m is the number of samples in time

  data = A(:); Ndata = length(data);

  // create a new vector
  [sim,vector] = ld_constvec(sim, ev, data );
  
  // vector extract test
//  [sim,index] = ld_const(sim, defaultevents, 2);  // max is Nsamples

  if special == "repeate" then
    [sim,one] = ld_const(sim, ev, 1);
    [sim, index] = ld_modcounter(sim, ev, in=one, initial_count=0, mod=Nsamples);
  else
    error("wrong special string. Should be one of ""repeate"", ...");
  end

//  [sim] = ld_printf(sim, ev, index, "index = ", 1);

  [sim, start_at] = ld_gain(sim, ev, index, Nvecplay);
  [sim, start_at] = ld_add_ofs(sim, ev, start_at, 1);

  [sim,outvec] = ld_vector_extract(sim, ev, in=vector, from=start_at, window_len=Nvecplay, vecsize=Ndata );
endfunction


function [sim,y] = ld_alternate( sim, ev, start_with_zero ) // PARSEDOCU_BLOCK
//
// %PURPOSE: generate an alternating sequence     
//
// y * - output
//
//
// [0, 1, 0, 1, 0, ... ], if start_with_zero == %T
// [1, 0, 1, 0, 1, ... ], if start_with_zero == %F
//
  z = poly(0,'z');    

  [sim,one] = ld_const(sim, ev, 1);

  [sim, fb] = libdyn_new_feedback();
  
  [sim, su ] = ld_add(sim, ev, list(fb, one), [-1,1] );
  [sim, del] = ld_ztf(sim, ev, su, 1/z); // a delay of one sample
  
  [sim] = libdyn_close_loop(sim, del, fb);
  
  y = del; 
  
  if (start_with_zero == %F) then
    [sim, y] = ld_not(sim, ev, y);
  end
    
endfunction

function [sim, out] = ld_detect_step_event(sim, ev, in, eps) // PARSEDOCU_BLOCK
    //
    // %PURPOSE: step detection block
    //
    // Detect jumps in the signal "in".
    // Everytime a jump occurs "out" is an impulse with the intensity of the
    // value after the jump i.e. if a signal steps from 1 to 2 there
    // will be an impulse out = 2
    // if no steps occur, out is zero
    //
    
    z = poly(0, 'z');
    
    [sim, i1] = ld_ztf(sim, ev, in, (z-1)/z ); // diff
    [sim, i2] = ld_abs(sim, ev, i1);
    
    [sim,event] = ld_compare_01(sim, ev, in=i2, thr=eps);
    [sim, out] = ld_mult(sim, ev, inp_list=list(event, in), muldiv1_list=[0, 0]);
    
endfunction

function [sim, out] = ld_detect_step_event2(sim, ev, in, eps) // PARSEDOCU_BLOCK
    //
    // %PURPOSE: step detection block
    //
    // Detect jumps in the signal "in".
    // Everytime a jump occurs "out" is an impulse with an intensity of 1,
    // else it is zero
    //
    
    z = poly(0, 'z');
    
    [sim, i1] = ld_ztf(sim, ev, in, (z-1)/z ); // diff
    [sim, i2] = ld_abs(sim, ev, i1);
    
    [sim,event] = ld_compare_01(sim, ev, in=i2, thr=eps);
    out = event;
    
endfunction

function [sim, reached] = ld_reference_reached(sim, ev, r, y, N, eps) // PARSEDOCU_BLOCK
    //
    // %PURPOSE: check wheter a reference value is reached 
    //
    // r * - Reference Signal (shall be constant)
    // y * - Signal to compare to the reference
    // N - After the condition of reaching the reference is true (and stays true)
    //     N time steps will be wait until setting reached to one. 
    // eps - the half tolerance band width
    //

  [sim, e] = ld_add(sim, ev, list(r,y), list(1,-1) );
  //[sim, i1] = ld_ztf(sim, ev, e, 1/(3+1) * (1 + z^(-1) + z^(-2) + z^(-3) ) );

  i1 = e;

  [sim, i3] = ld_abs(sim, ev, i1);
  [sim, i4] = ld_compare_01(sim, ev, in=i3,  thr=eps);
  [sim, i5] = ld_not(sim, ev, in=i4);
  
  [sim, resetto] = ld_const(sim, ev, 0);
  [sim, count] = ld_counter(sim, ev, count=i5, reset=i4, resetto, initial=0);

  [sim, reached] = ld_compare_01(sim, ev, in=count,  thr=N);
endfunction

function [sim, reached] = ld_greaterEpsForNSteps(sim, ev, in, thr, N) // PARSEDOCU_BLOCK
    //
    // %PURPOSE: return true, when input is greater a constant for more than N sampling steps
    //
    // in * - input signal
    // thr - threshold constant
    // N - integer
    //
    // If in is greater than thr for more than N time steps, reached = 1;
    // Otherwise reached is set to 0.
    
  [sim, i4] = ld_compare_01(sim, ev, in,  thr);
  [sim, i5] = ld_not(sim, ev, in=i4);
  
  [sim, resetto] = ld_const(sim, ev, 0);
  [sim, count] = ld_counter(sim, ev, count=i4, reset=i5, resetto, initial=0);

  [sim, reached] = ld_compare_01(sim, ev, in=count,  thr=N);
    
endfunction



function [sim] = ld_file_save_machine(sim, ev, in, cntrl, intype, insize, fname) // PARSEDOCU_BLOCK
//
// %PURPOSE: Start and stop saving of data to files
// 
//   in *+(size) - Data to write
//   cntrl * - if cntrl steps to 2 then saving is started; if it steps to 1 saving is stopped
//   intype - ORTD input type of data
//   size - amount of elements in the vector in
//   fname - string: Filename for saving
// 
// Note: The implementation of this function is a superblock using state machines
//       and the ld_savefile block. If you're interested take the source as an example.
// 
// 

    function [sim, outlist, active_state, x_global_kp1, userdata] = state_mainfn(sim, inlist, x_global, state, statename, userdata)
      // This function is called multiple times -- once for each state.
      // At runtime, these are three different nested simulations. Switching
      // between them represents state changing, thus each simulation 
      // represents a certain state.
      
//       printf("defining savemachine state %s (#%d) ... userdata(1)=%s\n", statename, state, userdata(1) );
      
      // define names for the first event in the simulation
      ev = 0; events = ev;

      // 
      dataToSave = inlist(1);
      switch = inlist(2);  [sim, switch] = ld_gain(sim, ev, switch, 1);


      // demultiplex x_global
      [sim, x_global] = ld_demux(sim, events, vecsize=1, invec=x_global);


      // The signals "active_state" is used to indicate state switching: A value > 0 means the 
      // the state enumed by "active_state" shall be activated in the next time step.
      // A value less or equal to zero causes the statemachine to stay in its currently active
      // state

      select state
	case 1 // state 1
	  active_state = switch;
	  [sim] = ld_printf(sim, ev, in=dataToSave, str="Pauseing Save", insize=DataLen);

	case 2 // state 2
	  [sim] = ld_savefile(sim, ev, fname, source=dataToSave, vlen=DataLen);
	  [sim] = ld_printf(sim, ev, in=dataToSave, str="Saveing", insize=DataLen);

	  active_state = switch;
      end

      // multiplex the new global states
      [sim, x_global_kp1] = ld_mux(sim, ev, vecsize=1, inlist=x_global);
      
      // the user defined output signals of this nested simulation
      outlist = list();
  endfunction

  DataLen = insize;


//   [sim] = ld_printf(sim, ev, cntrl, "cntrl", 1);

  [sim, cntrl] = ld_detect_step_event(sim, ev, in=cntrl, eps=0.2);

//   [sim] = ld_printf(sim, ev, cntrl, "Dcntrl", 1);


  // set-up two states represented by two nested simulations
  [sim, outlist, x_global, active_state,userdata] = ld_statemachine(sim, ev=defaultevents, ...
      inlist=list(in, cntrl), ..
      insizes=[insize,1], outsizes=[], ... 
      intypes=[intype ,ORTD.DATATYPE_FLOAT  ], outtypes=[], ...
      nested_fn=state_mainfn, Nstates=2, state_names_list=list("pause", "save"), ...
      inittial_state=1, x0_global=[1], userdata=list("hallo")  );


endfunction







// 
// Blocks, which C functions have not been move to the basic module yet, but the interfacing function
// 



// compare block. If input > thr: 
// optional_cmp_mode342 == 0: output = 1; else -1
// optional_cmp_mode342 == 1: output = 1; else 0
function [sim,bid] = libdyn_new_compare(sim, events, thr, optional_cmp_mode342)    
  if (exists('optional_cmp_mode342') ~= 1) then
    optional_cmp_mode342 = 0;
  end

  btype = 140;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [optional_cmp_mode342], [thr]);
endfunction


function [sim,y] = ld_compare(sim, events, in,  thr) // PARSEDOCU_BLOCK
//
// %PURPOSE: compare block. 
//   thr - constant
//   in * - signal
//   y *
// If in > thr: y = 1; else y = -1
// 
// Please check: Bug inside?
//

    [sim,blk] = libdyn_new_compare(sim, events, thr);
    [sim,blk] = libdyn_conn_equation(sim, blk, list(in,0));
    [sim,y] = libdyn_new_oport_hint(sim, blk, 0);    
endfunction

function [sim,y] = ld_compare_01(sim, events, in,  thr) // PARSEDOCU_BLOCK
//
// %PURPOSE: compare block. 
//   thr - constant
//   in - signal
//   y *
// If in > thr: y = 1; else y = 0
//

    [sim,blk] = libdyn_new_compare(sim, events, thr, 1); // mode = 1
    [sim,blk] = libdyn_conn_equation(sim, blk, list(in,0));
    [sim,y] = libdyn_new_oport_hint(sim, blk, 0);    
endfunction


















//
// Blocks
//


// FIXME: komische Funktion (noch gebraucht?)
function [sim,bid] = libdyn_new_blk_sum_pn(sim)
  btype = 10;
  [sim,oid] = libdyn_new_objectid(sim);
  id = oid; // id for this parameter set
  
  sim.parlist = new_irparam_elemet(sim.parlist, id, IRPAR_LIBDYN_BLOCK, [btype; oid], []);
endfunction


function [sim,bid] = libdyn_new_blk_gen(sim, events, symbol_name, ipar, rpar)
  btype = 5000;
  str = str2code(symbol_name);
  
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [length(str); str; ipar], [rpar]);
endfunction


// FIXME no port size checking
function [sim,bid] = libdyn_new_blk_sum(sim, events, c1, c2)
  btype = 12;
  
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [], [c1; c2]);
  
//  [sim,bid] = libdyn_new_blockid(sim);
//  id = bid; // id for this parameter set
//  
//  sim.parlist = new_irparam_elemet(sim.parlist, id, IRPAR_LIBDYN_BLOCK, [btype; bid], [c1; c2]);
endfunction


// FIXME no port size checking
function [sim,bid] = libdyn_new_blk_zTF(sim, events, H)
  btype = 30;
  bip = [ degree(H.num); degree(H.den) ];
  brp = [ coeff(H.num)'; coeff(H.den)' ];

  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [bip], [brp]);
  
//  [sim,bid] = libdyn_new_blockid(sim);
//  id = bid; // id for this parameter set
//  
//  
//  sim.parlist = new_irparam_elemet(sim.parlist, id, IRPAR_LIBDYN_BLOCK, [btype; bid; bip], [brp]);
endfunction

function [sim,bid] = libdyn_new_blk_const(sim, events, c1)
  btype = 40;
  [sim,bid] = libdyn_new_block(sim, events, btype, [], [c1], ...
                   insizes=[], outsizes=[1], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );
endfunction

function [sim,bid] = libdyn_new_blk_gain(sim, events, c)
  btype = 20;
  [sim,bid] = libdyn_new_block(sim, events, btype, [], [c], ...
                   insizes=[1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );
endfunction


function [sim,bid] = libdyn_new_blk_sat(sim, events, lowerlimit, upperlimit)
  btype = 50;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [], [lowerlimit, upperlimit]);
endfunction


//
// A switching Block
// inputs = [control_in, signal_in]
// if control_in > 0 : signal_in is directed to output 1; output_2 is set to zero
// if control_in < 0 : signal_in is directed to output 2; output_1 is set to zero
//
function [sim,bid] = libdyn_new_blk_switch(sim, events)
  btype = 60;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [], []);
endfunction

// Multiplication 
// d1, d2: multiplicate (=0) or divide (=1) corresponding input; need exactly 2 inputs
function [sim,bid] = libdyn_new_blk_mul(sim, events, d1, d2)
  btype = 70;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [d1, d2], []);
endfunction

// function generator
// at the moment only sinus (shape = ?)
// input port 1: amplitude
// input port 2: frequency
function [sim,bid] = libdyn_new_blk_fngen(sim, events, shape_)
  btype = 80;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [shape_], []);
endfunction
 
// serial to parallel
function [sim,bid] = libdyn_new_blk_s2p(sim, events, len)
  btype = 90;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [len], []);
endfunction

//
// Sample play block
//
// plays the sequence stored in r
// each time event 0 occurs the next value of r is put out
// sampling start either imediadedly (initial_play=1) or on event 1.
// Event 2 stops sampling and set ouput to last values (mute_afterstop = 0 and hold_last_values == 1) 
// or zero (mute_afterstop = 1)
//
function [sim,bid] = libdyn_new_blk_play(sim, events, r, initial_play, hold_last_value, mute_afterstop)
  if (exists('initial_play') ~= 1) then
    initial_play = 1;  
  end
  if (exists('hold_last_value') ~= 1) then
    hold_last_value = 0;  
  end
  if (exists('mute_afterstop') ~= 1) then
    mute_afterstop = 0;  
  end
  
  btype = 100;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [length(r), initial_play, hold_last_value, mute_afterstop], [r]);
endfunction


//
// A swich with two inputs and one output depending on a state. initial state=initial_state
// state 0 means the first input is feed through
// state 1 means the first input is feed through
//
// Switching occurs to state 0 if event 1 occurs
// Switching occurs to state 1 if event 2 occurs
// event 0 is the normal regular event
//

function [sim,bid] = libdyn_new_blk_2to1evsw(sim, events, initial_state)
  btype = 110;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [initial_state], []);
endfunction

// Lookup table: Inputs between lowerin and upperin will be mapped linear to the indices of table
//               The corresponsing element of table will be the output
function [sim,bid] = libdyn_new_blk_lkup(sim, events, lowerin, upperin, table)
  btype = 120;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [length(table)], [ lowerin, upperin, table(:)' ]);
endfunction


// Dump at max maxlen samples to file "filename"; start automatically if autostart == 1
function [sim,bid] = libdyn_new_blk_filedump(sim, events, filename, vlen, maxlen, autostart)
  btype = 130;
  fname = ascii(filename);
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [maxlen, autostart, vlen, length(fname), fname(:)'], []);
endfunction


// // delay of delay_len samples block
// function [sim,bid] = libdyn_new_delay(sim, events, delay_len)
//   btype = 150;
//   [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [delay_len], []);
// endfunction

function [sim,bid] = libdyn_new_flipflop(sim, events, initial_state)
  btype = 160;
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [initial_state], []);
endfunction











//////////////////////////////////////////////////////////
// fancy shortcuts, that *should* be used by the user
//////////////////////////////////////////////////////////

function [sim,c] = ld_const(sim, events, val) // PARSEDOCU_BLOCK
//
// %PURPOSE: A constant val
//

    [sim,c] = libdyn_new_blk_const(sim, events, val); // Instead of event a predefined initial event that only occurs once should be used
    [sim,c] = libdyn_new_oport_hint(sim, c, 0);    
endfunction

function [sim,sum_] = ld_sum(sim, events, inp_list, fak1, fak2) 
// FIXME obsolete
    [sim,sum_] = libdyn_new_blk_sum(sim, events, fak1, fak2);
    [sim,sum_] = libdyn_conn_equation(sim, sum_, inp_list);  
    [sim,sum_] = libdyn_new_oport_hint(sim, sum_, 0);    
endfunction

function [sim,sum_] = ld_add(sim, events, inp_list, fak_list) // PARSEDOCU_BLOCK
//
// %PURPOSE: Add signals (linear combination)
// inp_list = list( in1, in2 )  ; fak_list = [ c1, c2 ]
// sum_ = in1 * c1 + in2 * c2
//

    [sim,sum_] = libdyn_new_blk_sum(sim, events, fak_list(1), fak_list(2));
    [sim,sum_] = libdyn_conn_equation(sim, sum_, inp_list);  
    [sim,sum_] = libdyn_new_oport_hint(sim, sum_, 0);    
endfunction


function [sim,mul_] = ld_mul(sim, events, inp_list, muldiv1, muldiv2)
// %PURPOSE: Multiplication 
// muldiv1/2: multiplicate (=0) or divide (=1) corresponding input; need exactly 2 inputs
 // FIXME obsolete
    [sim,mul_] = libdyn_new_blk_mul(sim, events, muldiv1, muldiv2);
    [sim,mul_] = libdyn_conn_equation(sim, mul_, inp_list);  
    [sim,mul_] = libdyn_new_oport_hint(sim, mul_, 0);    
endfunction

function [sim,mul_] = ld_dot(sim, events, inp_list, muldiv1_list)
// %PURPOSE: Multiplication 
// muldiv1/2: multiplicate (=0) or divide (=1) corresponding input; need exactly 2 inputs
// inp_list = list( in1, in2 )  ; muldiv1_list = [ muldiv1, muldiv2 ]
    [sim,mul_] = libdyn_new_blk_mul(sim, events, muldiv1_list(1), muldiv1_list(2) );
    [sim,mul_] = libdyn_conn_equation(sim, mul_, inp_list);  
    [sim,mul_] = libdyn_new_oport_hint(sim, mul_, 0);    
endfunction

function [sim,mul_] = ld_mult(sim, events, inp_list, muldiv1_list) // PARSEDOCU_BLOCK
//
// %PURPOSE: Multiplication 
// muldiv1/2: multiplicate (=0) or divide (=1) corresponding input; need exactly 2 inputs
// inp_list = list( in1, in2 )  ; muldiv1_list = [ muldiv1, muldiv2 ]
//
    [sim,mul_] = libdyn_new_blk_mul(sim, events, muldiv1_list(1), muldiv1_list(2) );
    [sim,mul_] = libdyn_conn_equation(sim, mul_, inp_list);  
    [sim,mul_] = libdyn_new_oport_hint(sim, mul_, 0);    
endfunction


function [sim,gain] = ld_gain(sim, events, inp_list, gain) // PARSEDOCU_BLOCK
//
// %PURPOSE: A simple gain
//
//
  [inp] = libdyn_extrakt_obj( inp_list ); // compatibility

  [sim,gain] = libdyn_new_blk_gain(sim, events, gain);  
  [sim,gain] = libdyn_conn_equation(sim, gain, list(inp,0));
  [sim,gain] = libdyn_new_oport_hint(sim, gain, 0);    
endfunction

function [sim,sign_] = ld_sign(sim, events, inp_list, thr) // PARSEDOCU_BLOCK
//
// FIXME: OSOLETE FN: use ld_getsign instead
// 
// return the sign of the input sigal
// either 1 or -1
// 
// 
//
    [inp] = libdyn_extrakt_obj( inp_list ); // compatibility

    [sim,sign_] = libdyn_new_compare(sim, events, thr);
    [sim,sign_] = libdyn_conn_equation(sim, sign_, list(inp));
    [sim,sign_] = libdyn_new_oport_hint(sim, sign_, 0);
endfunction

function [sim,lkup] = ld_lkup(sim, events, inp_list, lower_b, upper_b, table)
// %PURPOSE: lookup table
//
// inp_list - input signal
// table - a vector of values to look up
// the input is mapped between lower_b and upper_b to the
// index within table
  [inp] = libdyn_extrakt_obj( inp_list ); // compatibility

    [sim,lkup] = libdyn_new_blk_lkup(sim, events, lower_b, upper_b, table);
    [sim,lkup] = libdyn_conn_equation(sim, lkup, list(inp));
    [sim,lkup] = libdyn_new_oport_hint(sim, lkup, 0);
endfunction
  
function [sim,fngen] = ld_fngen(sim, events, inp_list, shape_)
// %PURPOSE: function generator
// need cleanup
    [sim,fngen] = libdyn_new_blk_fngen(sim, events, shape_)
    [sim,fngen] = libdyn_conn_equation(sim, fngen, inp_list);
    [sim,fngen] = libdyn_new_oport_hint(sim, fngen, 0);
endfunction


// function [sim,delay] = ld_delay(sim, events, inp_list, delay_len)
//     [inp] = libdyn_extrakt_obj( inp_list ); // compatibility
// 
//     [sim,delay] = libdyn_new_delay(sim, events, delay_len)
//     [sim,delay] = libdyn_conn_equation(sim, delay, list(inp));
//     [sim,delay] = libdyn_new_oport_hint(sim, delay, 0);    
// endfunction

function [sim,y] = ld_ztf(sim, events, inp_list, H) // PARSEDOCU_BLOCK
//
// %PURPOSE: Time discrete transfer function
// H is give as a Scilab rational
//

  [inp] = libdyn_extrakt_obj( inp_list ); // compatibility

    [sim,tf] = libdyn_new_blk_zTF(sim, events, H);
    [sim,y] = libdyn_conn_equation(sim, tf, list(inp));
    [sim,y] = libdyn_new_oport_hint(sim, y, 0);    
endfunction

function [sim,y] = ld_sat(sim, events, inp_list, lowerlimit, upperlimit) // PARSEDOCU_BLOCK
//
// %PURPOSE: Saturation between lowerlimit and upperlimit
//


  [inp] = libdyn_extrakt_obj( inp_list ); // compatibility

    [sim,sat] = libdyn_new_blk_sat(sim, events, lowerlimit, upperlimit);
    [sim,y] = libdyn_conn_equation(sim, sat, list(inp));
    [sim,y] = libdyn_new_oport_hint(sim, y, 0);    
endfunction

function [sim,y] = ld_flipflop(sim, events, set0, set1, reset, initial_state) // PARSEDOCU_BLOCK
//
// %PURPOSE: A flip-flop (FIXME: more documentation needed)
//
// set0, set1, reset * - control of the flipflop
// initial_state - constant
//
    [sim,blk] = libdyn_new_flipflop(sim, events, initial_state);
    [sim,blk] = libdyn_conn_equation(sim, blk, list(set0,0, set1,0, reset,0)); // FIXME: remove ,0
    [sim,y] = libdyn_new_oport_hint(sim, blk, 0);    
endfunction




// function [sim,bid] = libdyn_new_printf(sim, events, str, insize) //REMOVE
//   btype = 170;
//   str = ascii(str);
//   [sim,bid] = libdyn_new_blk_generic(sim, events, btype, [insize, length(str), str(:)'], []);
// endfunction



//
// Some terminal color codes for usage with ld_printf
//

ORTD.termcode.red = ascii(27) + '[31m';
ORTD.termcode.green = ascii(27) + '[32m';
ORTD.termcode.yellow = ascii(27) + '[33m';
ORTD.termcode.blue = ascii(27) + '[34m';
ORTD.termcode.reset = ascii(27) + '[0m';


function [sim] = ld_printf(sim, events, in, str, insize) // PARSEDOCU_BLOCK
//
// %PURPOSE: Print data to stdout (the console)
//
// in *+(insize) - vectorial input signal
//
// str is a string that is printed followed by the signal vector in
// of size insize
//
// Hint: Apply colored printf's by using the predefined terminal color codes:
// 
// str = ORTD.termcode.red + "some colored text..." + ORTD.termcode.reset
// 
// instead of red there currently is: green, yellow, blue.
// 
// 
// 


  btype = 170;
  str = ascii(str);
//   [sim,blk] = libdyn_new_blk_generic(sim, events, btype, [insize, length(str), str(:)'], []);

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ insize, length(str), str(:)' ], rpar=[ ], ...
                   insizes=[ insize ], outsizes=[], ...
                   intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[]  );


  [sim,blk] = libdyn_conn_equation(sim, blk, list(in,0) );
endfunction





function [sim,out_1, out_2] = ld_switch(sim, events, cntrl, in) // PARSEDOCU_BLOCK
//
// %PURPOSE: A switching Block
// inputs = [control_in, signal_in]
// if control_in > 0 : signal_in is directed to output 1; output_2 is set to zero
// if control_in < 0 : signal_in is directed to output 2; output_1 is set to zero
//
    [sim,blk] = libdyn_new_blk_switch(sim, events);
    [sim,blk] = libdyn_conn_equation(sim, blk, list(cntrl,0, in,0));
    
    [sim,out_1] = libdyn_new_oport_hint(sim, blk, 0);
    [sim,out_2] = libdyn_new_oport_hint(sim, blk, 1);
endfunction


function [sim,y] = ld_play_simple(sim, events, r) // PARSEDOCU_BLOCK
//
// %PURPOSE: Simple sample play block
//
// plays the sequence stored in r
// each time event 0 occurs the next value of r is put out
//


  [sim,y] = libdyn_new_blk_play(sim, events, r, 1, 1, 0);
  [sim,y] = libdyn_new_oport_hint(sim, y, 0);    
endfunction



function [sim,save_]=libdyn_dumptoiofile(sim, events, fname, source) //OBSOLET
// %PURPOSE: Quick and easy dumping of signals to files in one line of code
// obsolete version
  [source] = libdyn_extrakt_obj( source ); // compatibility

  // source: a list with a block + a port
  [sim,save_] = libdyn_new_blk_filedump(sim, events, fname, 1, 0, 1);
  [sim,save_] = libdyn_conn_equation(sim, save_, list(source, 0) );
endfunction


function [sim,save_]=ld_dumptoiofile(sim, events, fname, source) //OBSOLET
//
// Quick and easy dumping of signals to files
// source - signal of size 1 (at the moment)
// fname - filename string
//
  [inp] = libdyn_extrakt_obj( source ); // compatibility

  // source: a list with a block + a port
  [sim,save_] = libdyn_new_blk_filedump(sim, events, fname, 1, 0, 1);
  [sim,save_] = libdyn_conn_equation(sim, save_, list(source) );
endfunction






// Interfacing functions are placed in this place




function [sim, out] = ld_startproc(sim, events, exepath, whentorun) // PARSEDOCU_BLOCK
// 
// %PURPOSE: Execute a sub process (EXPERIMENTAL)
//
// out * - output (unused)
// exepath - string: path to executable
// whentorun - 0 - Start process on blocks initialisation and stop on desruction; 
//             1 - Start process on activation in a statemachine and stop on reset
// 
// 

  printf("ld_startproc: starting %s\n", exepath);

 btype = 15300 + 0; // the same id you are giving via the "libdyn_compfnlist_add" C-function
 [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ 0, 0, 0, 0, whentorun,length(exepath),0, ascii(exepath)  ], rpar=[  ], ...
                  insizes=[], outsizes=[1], ...
                  intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );

//  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
 [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port


 [sim, out] = ld_gain(sim, events, out, 1); // FIXME: Otherwise the block above may not be initialised
endfunction

function [sim, out] = ld_startproc2(sim, events, exepath, chpwd, prio, whentorun) // PARSEDOCU_BLOCK
// 
// %PURPOSE: Execute a sub process with some more options (EXPERIMENTAL)
//
// out * - output
// exepath - string: path to executable
// chpwn - change current directory before running the process
// prio - Priority (set to zero for now)
// whentorun - 0 - Start process on blocks initialisation and stop on desruction; 
//             1 - Start process on activation in a statemachine and stop on reset
// 
// 

printf("ld_startproc: starting %s\n", exepath);

 btype = 15300 + 0; // the same id you are giving via the "libdyn_compfnlist_add" C-function
 [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ 0, 0, 0, prio, whentorun,length(exepath), length(chpwd), ascii(exepath), ascii(chpwd)  ], rpar=[  ], ...
                  insizes=[], outsizes=[1], ...
                  intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );

//  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
 [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port


 [sim, out] = ld_gain(sim, events, out, 1); // FIXME: Otherwise the block above may not be initialised
endfunction


// Interfacing functions are placed in this place


// ADJUST HERE: Add your interfacing functions here

// An example could be the following scilab function. PARSEDOCU_BLOCK is a hint for the documentation generator to generate documentation for this block
//function [sim, out] = ld_abs(sim, events, in) // PARSEDOCU_BLOCK
//// ADD SOME DOCUMENTATION HERE
//// abs - block
////
//// in * - input
//// out * - output
//// 
//// out = abs(in)
//// 
//
//  btype = 60001 + 7; // the same id you are giving via the "libdyn_compfnlist_add" C-function
//  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ <ADD INTEGERS HERE>  ], rpar=[ <ADD FLOATING POITN VALUES HERE> ], ...
//                   insizes=[1], outsizes=[1], ...
//                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );
//
//  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
//  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
//endfunction

// if you like to put strings as arguments for the block you could used ipar=[ length(str), ascii(str) ] for example and use "irpar_getstr" within C-Code for decoding (see irpar.h and irpar.c)
// Interfacing functions are placed in this place

// function [sim,out] = ld_constmat(sim, events, mat) // PARSEDOCU_BLOCK
// // 
// // a constant matrix
// // 
// // mat *+ - the matrix
// // 
//   btype = 69001 + 1;
//   [nr, nc] = size(mat);
//   mat_length = nr*nc + 2;
// 
//   ipar = [mat_length; nr; nc; 0];
//   rpar = [nr,nc];
//   for i=1:nr,
//     rpar=cat(2,rpar, mat(i,:))
//   end
//   rpar=rpar';
// 
//   [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
//                    insizes=[1], outsizes=[ mat_length ], ...
//                    intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );
//  
//   [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
// endfunction
// 

// function [sim,out] = ld_constmat(sim, events, mat) // PARSEDOCU_BLOCK
// // 
// // a constant matrix
// // 
// // mat *+ - the matrix
// // 
// 
//   [nr, nc] = size(mat);
//   //rpar = [nr,nc];
//   rpar = [];
//   for i=1:nr,
//     rpar=cat(2,rpar, mat(i,:))
//   end
//   //rpar=rpar';
// 
//   [sim,out] = ld_constvec(sim, defaultevents, rpar );
// endfunction

function [sim,out] = ld_constmat(sim, events, mat) // PARSEDOCU_BLOCK
// 
// %PURPOSE: a constant matrix
// 
// mat *+ - the matrix
// 

//   [nr, nc] = size(mat);
//   //rpar = [nr,nc];
//   rpar = zeros(1,nr*nc);
//   for i=1:nr,
// //     rpar=cat(2,rpar, mat(i,:))
//     rpar( nc*(i-1)+1 : nc*i ) = mat(i,:);
//   end
//   //rpar=rpar';

  tmp = mat';

  [sim,out] = ld_constvec(sim, events, tmp(:)' );
endfunction

function [sim,out] = ld_matmul(sim, events, left_matrix, left_matrix_size, right_matrix, right_matrix_size) // PARSEDOCU_BLOCK
// %PURPOSE: matrix multiplication of two matrices
// 
// out = left_matrix * right_matrix 
//
//    left_matrix *+(left_matrix_size)  - matrix signal
//    left_matrix_size ---> matrix size [nr,nc]
//
//    right_matrix *+(right_matrix_size)  - matrix signal
//    right_matrix_size ---> matrix size [nr,nc]
//    
  btype = 69001 + 3;

  if (length(left_matrix_size) ~= 2)  then
    printf("ld_matmul: Incorrect size definition left_matrix_size\n");
    error(".");
  end
  if (length(right_matrix_size) ~= 2)  then
    printf("ld_matmul: Incorrect size definition right_matrix_size\n");
    error(".");
  end

  lm_nr = left_matrix_size(1);
  lm_nc = left_matrix_size(2);
  rm_nr = right_matrix_size(1);
  rm_nc = right_matrix_size(2);


  if (lm_nc ~= rm_nr)  then
    printf("ld_matmul: Inconsistent multiplication -> %dx%d * %dx%d!\n", lm_nr, lm_nc, rm_nr, rm_nc);
    error(".");
  end

  l_in_size = (lm_nr*lm_nc);
  r_in_size = (rm_nr*rm_nc);
  out_size  = (lm_nr*rm_nc);

  ipar = [lm_nr; lm_nc; rm_nr; rm_nc]; rpar = [];

  

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[l_in_size, r_in_size], outsizes=[out_size], ...
                                     intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( left_matrix, right_matrix ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim,y] = ld_matvecmul_3d(sim, events, A, v) // PARSEDOCU_BLOCK
// %PURPOSE: Matrix multiplication against a vector for 3d-vectors
// 
// y = A * v, whereby A is 3x3 and v is a vector of size 3
// 
// 
// 

  [sim,y] = ld_matmul(sim, events, A, [3,3], v, [3,1] );
endfunction

function [sim, v1, v2, v3] = ld_getvecelements_3d(sim, events, v);
// %PURPOSE: Return the elements of a 3d-vector
// 
// v = [v1, v2, v3]

  [sim,elements] = ld_demux(sim, events, vecsize=3, invec=v);
  
  v1 = elements(1); v2 = elements(2); v3 = elements(3);
endfunction

function [sim, v] = ld_setvecelements_3d(sim, events, v1, v2, v3);
// %PURPOSE: Combine three vector elements to a 3d-vector
// 
// v = [v1, v2, v3]

  [sim, v] = ld_mux(sim, events, vecsize=3, inlist=list( v1, v2, v3) );
endfunction

function [sim, out, param] = ld_leastsquares(sim, events, inplist, veclen, n_param) // PARSEDOCU_BLOCK
// %PURPOSE: estimate n linear model parameters (a,b,c...) and compute yvec_hat.
// 
// yvec_hat = a * phi0 + b * phi1 + c * phi2 ...
// Estimation of a, b, c ... minimizes the sum of squared errors (\chi^2 = \sum_i (yvec - yvec_hat)^2).
// out = yvec_hat
//
// inplist* - list(phi0*, phi1*, phi2*, ..., phi(n_param-1)*, yvec*)
// + phi0* - input vector0 (The vectors phiX form the matrix of predictor variables.)
// + phi1* - input vector1 (The vectors phiX form the matrix of predictor variables.)
// + phi2* - input vector2 (The vectors phiX form the matrix of predictor variables.)
// + ...
// + phi(n_param-1)* - input vector(n_param-1) (The vectors phiX form the matrix of predictor variables.)
// + yvec* - the original vector
//
// veclen - the number of observations which have been made (the length of each vector phiX, yvec and yvec_hat)
// n_param - the number of vectors belonging to Phi as well as the number of parameters in param*.
// out* = yvec_hat (the best prediction of yvec)
// param* - [a, b, c, ...]
//
// NOTICE: ld_leastsquares is not available by default since you need to include the GSL - GNU Scientific Library.
// Please follow the instructions which have been testet under Ubuntu 12.04. LTS:
//
// - install the required library with the following command or with a package-manager of your choice:
// 'sudo apt-get install libgsl0-dev'
//
// - change to the directory /YOUR_PATH/openrtdynamics/trunk/modules/matrix/ and rename the file 'LDFLAGS.disabled'
// to 'LDFLAGS'
//
// - open the file 'matrix.c' which is located in the same directory with an editor of your choice and change the
// line '#define GSL_INCLUDED 0' to '#define GSL_INCLUDED 1' - then save and close the file.
//
// - change to the directory /YOUR_PATH/openrtdynamics/trunk/ and type run 'make clean', 'make' and 'sudo make install'
//
// - reload the toolbox in scilab and you should be able to use it.
//    
  btype = 69001 + 4;

  ipar = [veclen, n_param]; rpar = [];
  
  insizelist = [];
  intypelist = [];
  
  for j=1:(n_param + 1) // yvec is part of the list as well
    insizelist = [insizelist, veclen];
    intypelist = [intypelist, ORTD.DATATYPE_FLOAT];
  end
    

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=insizelist, outsizes=[veclen, n_param], ...
                                     intypes=intypelist, ...
                                     outtypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, inplist );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
  [sim,param] = libdyn_new_oport_hint(sim, blk, 1);   // 0th port
endfunction




//
//    Copyright (C) 2010, 2011  Christian Klauer
//
//    This file is part of OpenRTDynamics, the Real Time Dynamic Toolbox
//
//    OpenRTDynamics is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OpenRTDynamics is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with OpenRTDynamics.  If not, see <http://www.gnu.org/licenses/>.
//





// Interfacing functions are placed in this place


function [sim,bid] = libdyn_new_muparser(sim, events, Nin, Nout, str, float_param)
  btype = 11001;
  str = ascii(str);
  nparam = length(float_param);
  
  ipar = [Nin, Nout, nparam, length(str), str(:)'];
  rpar = [float_param(:)];
  
  [sim,bid] = libdyn_new_blk_generic(sim, events, btype, ipar, rpar);
endfunction

function [sim,out] = ld_muparser(sim, events, inlist, str, float_param) // PARSEDOCU_BLOCK
// 
// Evaluation of math formula
// 
// inlist - list() of signals of length 1
// str - string containing the formula
// float_param - vector of parameters
// 
// Within str "u1", "u2", ... refer to inputs and
//            "c1", "c2", ... refer to the parameters within float_param
// 
  Nin = length(inlist);
  Nout = 1;

  [sim,blk] = libdyn_new_muparser(sim, events, Nin, Nout, str, float_param);
  [sim,blk] = libdyn_conn_equation(sim, blk, inlist);
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


//
// Use: http://help.scilab.org/docs/5.3.2/en_US/strsubst.html
//


//parNames = ['par1', 'par2']; par = [ 0.2. 0.4 ];
//inNames = ['u', 'e' ]; inlist = list(x,x);
//str = 'sin(par1) + par2';
//

function [sim,out] = ld_muparser_subst(sim, events, inlist, str, par, inNames, parNames) // PARSEDOCU_BLOCK
// 
// Bug within this scilab function
// 
// subtitution: haveing names like "phi" and "phi_u" does not work
//              names "u1", "u2", ... and "c1", "c2", ... also does not work right now
// 
// 
// 

  Nin = length(inlist);
  Nout = 1;
  
  inNames = inNames(:);
  parNames = parNames(:);
  
  [ tmp1, tmp2 ] = size(inNames);
  NinNames = max(tmp1, tmp2);

  [ tmp1, tmp2 ] = size(parNames);
  NparNames = max(tmp1, tmp2);

//printf("--------- BEGIN MUPARSER BLOCK -----------\n");  

//  pause;

  if length(inlist) ~= NinNames then
      printf("inNames and inlist have to have equal length (%d != %d)\n", length(inlist), NinNames);
      error(".");
  end
  if length(par) ~= NparNames then
      printf("parNames and par have to have equal length (%d != %d)\n", length(par), NparNames);
      error(".");
  end

  // substitute variable names for the inputs
  i = 1;
  while i <= Nin 
    if typeof(inNames(i)) ~= 'string' then
      error("ld_muparser2: Did not find a string in invar");
    end


    if libdyn_is_ldobject(inlist(i)) == %F then
      error("mu_parser2: Did not find a libdyn object at some entry within inlist");
    end


    // replace the variable name
    replacement = 'u' + string(i);
    //printf("subst %s with %s\n", inNames(i), replacement);
    str = strsubst( str, inNames(i), replacement );
    //printf("result is " + str + "\n");

    i = i + 1;
  end
  
  // substitute variable names for the parameters
  i = 1;
  while i <= length(par) 
    if typeof(parNames(i)) ~= 'string' then
      error("ld_muparser2: Did not find a sting in parNames");
    end


    if typeof(par(i)) ~= 'constant' then
      error("mu_parser2: Did not find a skalar");
    end

    replacement = 'c' + string(i);
  //  printf("subst %s with %s\n", parNames(i), replacement);
    str = strsubst( str, parNames(i), replacement );


    i = i + 1;
  end


//  printf("The expression is now: %s\n", str);


  [sim,out] = ld_muparser(sim, events, inlist, str, par);
endfunction


// 
// 
// 
// 
// 
// function [sim,out] = ld_muparser2(sim, events, inlist, str, invar, par)
//   // 
//   Nin = length(inlist);
//   Nout = 1;
// 
//   parlist = new_irparam_set();
// 
// 
//   inlist = list();
// 
//   n_in = 0;
//   i = 1;
//   //i_ = 1:length(invar);
//   while i <= length(invar) 
//     if typeof(invar(i)) ~= 'string' then
//       error("ld_muparser2: Did not find a sting in invar");
//     end
// 
//     parlist = new_irparam_elemet_ivec(parlist, invar(i), 100+n_in); // save name of input variable
// 
//     n_in = n_in + 1;
//     i = i + 1;
// 
// 
//     if libdyn_is_ldobject(invar(i)) == %F then
//       error("mu_parser2: Did not find a libdyn object in invar");
//     end
// 
//     inlist(n_in) = invar(i);
// 
//     i = i + 1;
// 
//   end
//   
// 
//   pnr = 0;
//   i = 1;
//   //i_ = 1:length(invar);
//   while i <= length(par) 
//     if typeof(par(i)) ~= 'string' then
//       error("ld_muparser2: Did not find a sting in invar");
//     end
// 
//     parlist = new_irparam_elemet_ivec(parlist, ascii(par(i)), 10000+pnr); // save name of parameter
// 
//     pnr = pnr + 1;
//     i = i + 1;
// 
// 
//     if  typeof(par(i)) ~= 'constant' then
//       error("mu_parser2: Did not find a skalar");
//     end
// 
//     parlist = new_irparam_elemet_rvec(parlist, par(i), 20000+pnr); // save parameter value
//     
// 
//     i = i + 1;
// 
//   end
// 
// 
//   // store expression
//   str = ascii(str);
//   parlist = new_irparam_elemet_ivec(parlist, str, 10); 
// 
//   // combine ir parameters
//   blockparam = combine_irparam(parlist);
// 
// 
//   // block parameters
//   ipar = [Nin, Nout, blockparam.ipar];
//   rpar = [blockparam.rpar];
// 
//   btype = 11002; // block type id
//   
//   [sim,bid] = libdyn_new_blk_generic(sim, events, btype, ipar, rpar);
// 
// 
//   [sim,blk] = libdyn_conn_equation(sim, blk, inlist); // connect all imputs
//   [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
// endfunction
// 
// 


// Interfacing functions are placed in this place




function [sim, outlist, computation_finished] = ld_simnest(sim, ev, inlist, insizes, outsizes, intypes, outtypes, fn_list, dfeed, asynchron_simsteps, switch_signal, reset_trigger_signal  ) // PARSEDOCU_BLOCK
// 
// %PURPOSE: create one (or multiple) nested libdyn simulation within a normal libdyn block it is possible to switch between them by an special input signal
//
// INPUTS: 
//
// switch_signal: signal used to switch between different nested simulations
// reset_trigger_signal: when 1 the current simulation is reset (sync)
//                       OR when 1 the current simulation is triggered (async)
// inlist - list( ) of input signals to the block, that will be forwarded to the nested simulation(s)
//
// PARAMETERS:
// 
// ev - events to be forwarded to the nested simulation
// insizes - input ports configuration
// outsizes - output ports configuration
// intypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// outtypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// fn_list - list( ) of scilab functions defining sub-schematics
// dfeed - the block containing all sub-schematics is configured with dfeed
// asynchron_simsteps - if == 1 one simulation steps will be simulated in a thread
//                     when finished the result becomes available to the blocks outports
//                      if == 2 the simulation will be simulated in a thread and can be synchronised
//                      by the subsimulation itselft through synchronisation blocks (e.g. ld_synctimer)
//                     if == 0 the nested simulation runns synchronous to the upper level simulation. 
//                     (i.e. no thread is started)
// 
// OUTPUTS:
// 
// outlist - list( ) of output signals
// computation_finished - optional and only meanful if asynchron_simsteps > 0 (means async computation)
// 


  parlist = new_irparam_set();

  N1 = length(insizes);
  N2 = length(outsizes);
  N3 = length(intypes);
  N4 = length(outtypes);

  // check input signals for correctness
  try 
    libdyn_check_object(sim, switch_signal);
  catch
    error("Input signal switch_signal is not correct");
  end

  try
    libdyn_check_object(sim, reset_trigger_signal);
  catch
    error("Input signal reset_trigger_signal is not correct");
  end
    
  
  // check for sizes
  // ...
  if (length(inlist) ~= N1) then
    error("length inlist invalid or length of insizes invalid\n");
  end

  if N4 ~= N2 then
    error("length of outsizes invalid\n");
  end

  if N1 ~= N3 then
    error("length of intypes invalid\n");
  end


   parlist = new_irparam_elemet_ivec(parlist, insizes, 10); 
   parlist = new_irparam_elemet_ivec(parlist, outsizes, 11); 
   parlist = new_irparam_elemet_ivec(parlist, intypes, 12); 
   parlist = new_irparam_elemet_ivec(parlist, outtypes, 13); 


  if (length(dfeed) ~= 1) then
    error("dfeed should be of size 1\n");
  end

  if (length(asynchron_simsteps) ~= 1) then
    error("asynchron_simsteps should be of size 1\n");
  end


  Nsimulations = length(fn_list);
   parlist = new_irparam_elemet_ivec(parlist, [Nsimulations, dfeed, asynchron_simsteps], 20); 



  // Go through all schematics
  // and set them up
  // finially they are stored within an irpar structure under different irpar ids
  irpar_sim_idcounter = 900;

  for i = 1:Nsimulations

    fn = fn_list(i);
    //    [sim_container_irpar, sim] = libdyn_setup_schematic(fn, insizes, outsizes, intypes, outtypes); // outtypes and intypes are not handled at the moment
    [sim_container_irpar, nested_sim] = libdyn_setup_schematic(fn, insizes, outsizes); 

    // pack simulations into irpar container with id = 901
    parlist = new_irparam_container(parlist, sim_container_irpar, irpar_sim_idcounter);

    // increase irpar_sim_idcounter so the next simulation gets another id
    irpar_sim_idcounter = irpar_sim_idcounter + 1;
  end



//   // combine ir parameters
   blockparam = combine_irparam(parlist);


  // blockparam.ipar and blockparam.rpar now contain the blocks parameters

  // set-up the nested block

  btype = 15001;
//   [sim,blk] = libdyn_new_blk_generic(sim, ev, btype, [blockparam.ipar], [blockparam.rpar] );
  [sim,blk] = libdyn_new_block(sim, ev, btype, ipar=[blockparam.ipar], rpar=[blockparam.rpar], ...
                       insizes=[insizes(:)' ,1,1], outsizes=[outsizes(:)', 1], ...
                       intypes=[ones(insizes(:)') * ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                       outtypes=[ones(outsizes(:)') * ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT]  );


// 
  // add switch and reset input signals
  blocks_inlist = inlist;
  blocks_inlist($+1) = switch_signal;

  if asynchron_simsteps == 0 then
    // connect reset input
    blocks_inlist($+1) = reset_trigger_signal; //reset_signal;
  else
    // connect trigger input
    blocks_inlist($+1) = reset_trigger_signal; //trigger_signal;
  end

  // connect all inputs
  printf("ld_simnest: connection of inputs\n");
  try
    [sim,blk] = libdyn_conn_equation(sim, blk, blocks_inlist );
  catch
    error("ld_simnest: One of the input signals in inlist could not be connected");
  end
 
  // connect all outputs
  Noutp = length(outsizes);

  outlist = list();
  for i = 0:(Noutp-1)
    [sim,out] = libdyn_new_oport_hint(sim, blk, i);   // ith port
    outlist(i+1) = out;
  end

  if (asynchron_simsteps > 0) then
    [sim,computation_finished] = libdyn_new_oport_hint(sim, blk, Noutp);   // the last port
  else
    computation_finished = 0; // dummy value
  end

endfunction

function [sim, outlist, computation_finished, userdata] = ld_simnest2(sim, ev, inlist, insizes, outsizes, intypes, outtypes, nested_fn, Nsimulations, dfeed, asynchron_simsteps, switch_signal, reset_trigger_signal, userdata, simnest_name  )  // PARSEDOCU_BLOCK
// 
// %PURPOSE: create one (or multiple) nested libdyn simulation within a normal libdyn block
//                It is possible to switch between them by an special input signal
//                Replacement schematics can be generated by the "ld_simnest2_replacement" function.
//                The "ld_nested_exchffile"-block can be used to load the schematic into the controller
//
// INPUTS: 
//
// switch_signal: signal used to switch between different nested simulations
// reset_trigger_signal: when 1 the current simulation is reset (sync)
//                       OR when 1 the current simulation is triggered (async)
// inlist - list( ) of input signals to the block, that will be forwarded to the nested simulation(s)
//
// PARAMETERS:
// 
// ev - events to be forwarded to the nested simulation
// insizes - input ports configuration
// outsizes - output ports configuration
// intypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// outtypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// nested_fn - scilab function defining sub-schematics
// Nsimulations - number of simulations to switch inbetween
// dfeed - the block containing all sub-schematics is configured with dfeed
// asynchron_simsteps - if > 0 asynchron_simsteps steps will be simulated in a thread
//                     when finished the result becomes available to the blocks outports
//                     if == 0 the nested simulation runns synchronous to the upper level simulation.
// switch_signal 
// reset_trigger_signal
// userdata - arbitrary user date
// simnest_name - a string name with which the nested is refered.
// 
// OUTPUTS:
// 
// outlist - list( ) of output signals
// computation_finished - optional and only meanful if asynchron_simsteps > 0 (means async computation)
// 


  parlist = new_irparam_set();

  N1 = length(insizes);
  N2 = length(outsizes);
  N3 = length(intypes);
  N4 = length(outtypes);

  // check for sizes
  // ...
  if (length(inlist) ~= N1) then
    error("length inlist invalid or length of insizes invalid\n");
  end

  if N4 ~= N2 then
    error("length of outsizes invalid\n");
  end

  if N1 ~= N3 then
    error("length of intypes invalid\n");
  end


   parlist = new_irparam_elemet_ivec(parlist, insizes, 10); 
   parlist = new_irparam_elemet_ivec(parlist, outsizes, 11); 
   parlist = new_irparam_elemet_ivec(parlist, intypes, 12); 
   parlist = new_irparam_elemet_ivec(parlist, outtypes, 13); 


  if (length(dfeed) ~= 1) then
    error("dfeed should be of size 1\n");
  end

  if (length(asynchron_simsteps) ~= 1) then
    error("asynchron_simsteps should be of size 1\n");
  end



   parlist = new_irparam_elemet_ivec(parlist, [Nsimulations, dfeed, asynchron_simsteps], 20); 
   parlist = new_irparam_elemet_ivec(parlist, ascii(simnest_name), 21); 



  // Go through all schematics
  // and set them up
  // finially they are stored within an irpar structure under different irpar ids
  irpar_sim_idcounter = 900;

  for i = 1:Nsimulations

    //    [sim_container_irpar, sim] = libdyn_setup_schematic(fn, insizes, outsizes, intypes, outtypes); // outtypes and intypes are not handled at the moment
    //[sim_container_irpar, nested_sim] = libdyn_setup_schematic(fn, insizes, outsizes); 
    [sim_container_irpar, nested_sim, userdata] = libdyn_setup_sch2(nested_fn, insizes, outsizes,  intypes, outtypes, list(i, userdata)); 

    // pack simulations into irpar container with id = 901
    parlist = new_irparam_container(parlist, sim_container_irpar, irpar_sim_idcounter);

    // increase irpar_sim_idcounter so the next simulation gets another id
    irpar_sim_idcounter = irpar_sim_idcounter + 1;
  end



//   // combine ir parameters
   blockparam = combine_irparam(parlist);


  // blockparam.ipar and blockparam.rpar now contain the blocks parameters

  // set-up the nested block

  btype = 15001;
//   [sim,blk] = libdyn_new_blk_generic(sim, ev, btype, [blockparam.ipar], [blockparam.rpar] );
  [sim,blk] = libdyn_new_block(sim, ev, btype, ipar=[blockparam.ipar], rpar=[blockparam.rpar], ...
                       insizes=[insizes(:)' ,1,1], outsizes=[outsizes(:)', 1], ...
                       intypes=[ones(insizes(:)') * ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                       outtypes=[ones(outsizes(:)') * ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT]  );


// 
  // add switch and reset input signals
  blocks_inlist = inlist;
  blocks_inlist($+1) = switch_signal;

  if asynchron_simsteps == 0 then
    // connect reset input
    blocks_inlist($+1) = reset_trigger_signal; //reset_signal;
  else
    // connect trigger input
    blocks_inlist($+1) = reset_trigger_signal; //trigger_signal;
  end

  // connect all inputs
  [sim,blk] = libdyn_conn_equation(sim, blk, blocks_inlist );
 
  // connect all outputs
  Noutp = length(outsizes);

  outlist = list();
  for i = 0:(Noutp-1)
    [sim,out] = libdyn_new_oport_hint(sim, blk, i);   // ith port
    outlist(i+1) = out;
  end

  if (asynchron_simsteps > 0) then
    [sim,computation_finished] = libdyn_new_oport_hint(sim, blk, Noutp);   // the last port
  else
    computation_finished = 0; // dummy value
  end

endfunction


//   // FIXME: change PARSEDOCU_BLOCK to   // PARSEDOCU_FUNCTION
function [par, userdata] = ld_simnest2_replacement( insizes, outsizes, intypes, outtypes, nested_fn, userdata, N  )   // PARSEDOCU_BLOCK
// 
// %PURPOSE: create schematics that can be used as an online exchangeable simulation for nested simulations set-up using the ld_nested2 block
//
// The "ld_nested_exchffile"-block can be used to load the schematic into the controller
//
// INPUTS: 
//
// switch_signal: signal used to switch between different nested simulations
// reset_trigger_signal: when 1 the current simulation is reset (sync)
//                       OR when 1 the current simulation is triggered (async)
// inlist - list( ) of input signals to the block, that will be forwarded to the nested simulation(s)
//
// PARAMETERS:
// 
// insizes - input ports configuration
// outsizes - output ports configuration
// intypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// outtypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// nested_fn - scilab function defining sub-schematics
// N - FIXME ???
// 
// OUTPUTS:
//
// par - irpar data set. par.ipar and par.rpar contain the integer and double parameters list
// 


  parlist = new_irparam_set();

  N1 = length(insizes);
  N2 = length(outsizes);
  N3 = length(intypes);
  N4 = length(outtypes);

  // check for sizes
  // ...

  if N4 ~= N2 then
    error("length of outsizes invalid\n");
  end

  if N1 ~= N3 then
    error("length of intypes invalid\n");
  end

  if length(N) ~= 1 then
    error("N must be scalar\n");
  end

   parlist = new_irparam_elemet_ivec(parlist, insizes, 10); 
   parlist = new_irparam_elemet_ivec(parlist, outsizes, 11); 
   parlist = new_irparam_elemet_ivec(parlist, intypes, 12); 
   parlist = new_irparam_elemet_ivec(parlist, outtypes, 13); 



   parlist = new_irparam_elemet_ivec(parlist, [ -1, -1, -1, getdate("s"), -1, -1, getdate() ], 21); 
   parlist = new_irparam_elemet_ivec(parlist, [ ascii("ORTD-replacement-schematic") ], 22); 



  // Go through all schematics
  // and set them up
  // finially they are stored within an irpar structure under different irpar ids
  irpar_sim_idcounter = 100;

  for i = N

    //    [sim_container_irpar, sim] = libdyn_setup_schematic(fn, insizes, outsizes, intypes, outtypes); // outtypes and intypes are not handled at the moment
    //[sim_container_irpar, nested_sim] = libdyn_setup_schematic(fn, insizes, outsizes); 
//  printf("***\n");
//  pause;
    [sim_container_irpar, nested_sim, userdata] = libdyn_setup_sch2(nested_fn, insizes, outsizes, intypes, outtypes, list(i, userdata) ); 

    // pack simulations into irpar container with id = 901
    parlist = new_irparam_container(parlist, sim_container_irpar, irpar_sim_idcounter);

    // increase irpar_sim_idcounter so the next simulation gets another id
    irpar_sim_idcounter = irpar_sim_idcounter + 1;
  end



//   // combine ir parameters
   par = combine_irparam(parlist);
  // blockparam.ipar and blockparam.rpar now contain the blocks parameters

//   ipar = blockparam.ipar;
//   rpar = blockparam.rpar;

endfunction



function [sim, outlist, userdata ] = LD_STATEMACHINE_MAIN(sim, inlist, userdata)
  state = userdata(1);
  userdata_nested = userdata(2);
  fn = userdata(3);
  statename = userdata(4);  
  Nin_userdata = userdata(5);  
  Nout_userdata = userdata(6); 
  Nevents = userdata(7);
//  Nevents = 1;
  
  //pause;
    
  printf("Nested wrapper function: state=%s (#%d) Nin_userdata=%d Nout_userdata=%d, %d events are forwarded\n", statename, state, Nin_userdata, Nout_userdata, Nevents);
    
  x_global = inlist(Nin_userdata+1); // the global states
  
  // call the actual function
  [sim, outlist_inner, active_state, x_global_kp1, userdata] = fn(sim, inlist, x_global, state, statename, userdata_nested);


  if length(outlist_inner) ~= Nout_userdata then
      printf("Nested wrapper function: your provided function returns more or less outputs in outlist. Expecting %d but there are %d\n", Nout_userdata, length(outlist_inner));
      error(".");
  end

  outlist = list();

  N = length(outlist_inner);
  for i = 1:N
      outlist($+1) = outlist_inner(i);
  end
  outlist($+1) = active_state;
  outlist($+1) = x_global_kp1

//  outlist = list(, switch_event, global_states);
endfunction


function [sim, outlist, x_global, active_state, userdata] = ld_statemachine(sim, ev, inlist, insizes, outsizes, intypes, outtypes, nested_fn, Nstates, state_names_list, inittial_state, x0_global, userdata  ) // PARSEDOCU_BLOCK
// 
// %PURPOSE: A statemachine

//  create one (or multiple) nested libdyn simulation within a normal libdyn block
//                    it is possible to switch between them by an special signal. By this the
//                    nested simulations can be interpreted as a state machine, whereby each
//                    simulation describes a state. Additionally, global states (a vector of doubles)
//                    can be shared accross the different nested simulations.
//
// EXAMPLE: examples/state_machine.sce (probpably tells more then this reference)
//
// INPUTS: 
//
// inlist - list( ) of input signals to the block, that will be forwarded to the nested simulation(s)
//
// PARAMETERS:
// 
// ev - events to be forwarded to the nested simulation. The first one is also used by this nesting block as activation event
// insizes - input ports configuration
// outsizes - output ports configuration
// intypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// outtypes - ignored for now, put ORTD.DATATYPE_FLOAT for each port
// nested_fn - list( ) of one! scilab functions defining sub-schematics
// Nstates - number of simulations to switch inbetween
// state_names_list - list() of strings - one for each state
// inittial_state - number of the inittial state. counting starts at 1
// x0_global - inittial state of x_global
// userdata - anythig - feed to the schematic_fn
// 
// OUTPUTS:
// 
// outlist - list( ) of output signals
// x_global - signal of the states x_global
// active_state - signal with the number of the currently active state
// 
// PROTOTYPE FOR nested_fn:
// 
// [sim, outlist, active_state, x_global_kp1, userdata] = state_mainfn(sim, inlist, x_global, state, statename, userdata)
//
// inlist - the same signals as feed to ld_statemachine
// x_global - vector signal of the global states
// state - constant: number of the state to define
// statename - string: name of the state to define
// userdata - a custom variable as feed to ld_statemachine
//
// RETURS:
//
// outlist - 
// active_state - signal describing the active state. If this is a singal containing -1 no state swicth occurs
// x_global_kp1 - vectorial signal of the new global states 
//



  parlist = new_irparam_set();

  N1 = length(insizes);
  N2 = length(outsizes);
  N3 = length(intypes);
  N4 = length(outtypes);

  NGlobStates = length(x0_global);
  N_datainports = length(insizes);
  N_dataoutports = length(outsizes);


  // check for sizes
  // ...
  if (length(inlist) ~= N1) then
    error("length inlist invalid or length of insizes invalid\n");
  end

  if N4 ~= N2 then
    error("length of outsizes invalid\n");
  end

  if N1 ~= N3 then
    error("length of intypes invalid\n");
  end


  // io for the nested simulation (this is not eq to the outer blocks io)

   nested_insizes = [ insizes(:)', NGlobStates ];
   nested_outsizes = [ outsizes(:)', 1, NGlobStates];
   nested_intypes = [ intypes(:)', ORTD.DATATYPE_FLOAT];
   nested_outtypes = [ outtypes(:)', ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT ];

   parlist = new_irparam_elemet_ivec(parlist, nested_insizes, 10); 
   parlist = new_irparam_elemet_ivec(parlist, nested_outsizes, 11);  // datasigs, active_state, x_global
   parlist = new_irparam_elemet_ivec(parlist, nested_intypes, 12); 
   parlist = new_irparam_elemet_ivec(parlist, nested_outtypes, 13); 



   // parameters
   parlist = new_irparam_elemet_ivec(parlist, [Nstates, NGlobStates, inittial_state], 20); 
   parlist = new_irparam_elemet_rvec(parlist, [x0_global], 21);  // inittial global state



  // Go through all schematics
  // and set them up
  // finially they are stored within an irpar structure under different irpar ids
  irpar_sim_idcounter = 900;

  for i = 1:Nstates

    // the parameters for the wrapper scilab function
    wrapper_fn_arg = list(i, userdata, nested_fn, state_names_list(i), N_datainports, N_dataoutports, length(ev) );

    // create a nested simulation
    [sim_container_irpar, nested_sim, userdata] = libdyn_setup_sch2(LD_STATEMACHINE_MAIN, insizes=nested_insizes, ...
                           outsizes=nested_outsizes, nested_intypes, nested_outtypes, wrapper_fn_arg); 

    // pack simulations into irpar container with id = 901
    parlist = new_irparam_container(parlist, sim_container_irpar, irpar_sim_idcounter);

    // increase irpar_sim_idcounter so the next simulation gets another id
    irpar_sim_idcounter = irpar_sim_idcounter + 1;
  end



//   // combine ir parameters
   blockparam = combine_irparam(parlist);


  // blockparam.ipar and blockparam.rpar now contain the blocks parameters

  // set-up the nested block



  btype = 15002;
//   [sim,blk] = libdyn_new_blk_generic(sim, ev, btype, [blockparam.ipar], [blockparam.rpar] );
  [sim,blk] = libdyn_new_block(sim, ev, btype, ipar=[blockparam.ipar], rpar=[blockparam.rpar], ...
                       insizes=[insizes(:)'], outsizes=[outsizes(:)', 1, NGlobStates], ...
                       intypes=[intypes(:)'], ...
                       outtypes=[outtypes(:)', ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT]  );


// 
  // add switch and reset input signals
  blocks_inlist = inlist;

  // connect all inputs
  [sim,blk] = libdyn_conn_equation(sim, blk, blocks_inlist );
 
  // connect all outputs
  Noutp = length(outsizes);

  outlist = list();
  for i = 0:(Noutp-1)
    [sim,out] = libdyn_new_oport_hint(sim, blk, i);   // ith port
    outlist(i+1) = out;
  end

  [sim,active_state] = libdyn_new_oport_hint(sim, blk, Noutp);   // the second last port
  [sim,x_global] = libdyn_new_oport_hint(sim, blk, Noutp+1);   // the last port

endfunction



function [sim, out] = ld_nested_exchffile(sim, events, compresult, slot, fname, simnest_name) // PARSEDOCU_BLOCK
//
// %PURPOSE: Online exchange of a nested simulation via loading *[ir].par files
//
// NOTE: May delay realtime behaviour - use during a asynchronous nested simulation
//
// compresult - signal
// slot - signal
// fname - the filename without ".[ir]par"-ending
// simnest_name - the string which referes to the nested simulation (as given to "ld_simnest2")
// 
//

  ifname = fname + ".ipar";
  rfname = fname + ".rpar";

  btype = 15003;
  [sim,blk] = libdyn_new_block(sim, events, btype, [0,0,0, length(ifname), length(rfname), length(simnest_name), ascii(ifname), ascii(rfname), ascii(simnest_name) ], [ ], ...
                   insizes=[1, 1], outsizes=[1], ...
                   intypes=[ORTD.DATATYPE_FLOAT, ORTD.DATATYPE_FLOAT], ...
                   outtypes=[ORTD.DATATYPE_FLOAT]  );

  [sim,blk] = libdyn_conn_equation(sim, blk, list(compresult, slot) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, outvec] = ld_survivereset(sim, events, invec, initvec) // PARSEDOCU_BLOCK
// %PURPOSE: Keep stored data through a simulation reset (EXPERIMENTAL FOR NOW)
//
// in *+(vecsize) - input
// out *+(vecsize) - output
// 
// Prior to a simulation reset the input is stored into memory.
// After the reset the data is available again via the output
// Initially the output is set to initvec
// 
//    

  if (length(initvec) < 1) then
    error("length(initvec) < 1 !");
  end

  btype = 15004;	
  ipar = [length(initvec); 0]; rpar = [initvec];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                                     insizes=[length(initvec)], outsizes=[length(initvec)], ...
                                     intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[ORTD.DATATYPE_FLOAT] );
 
  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list( in ) );

  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction







// 
// A higher level for defining online replaceable schematics using callback functions
// 


function [sim, outlist, userdata, replaced] = ld_ReplaceableNest(sim, ev, inlist, trigger_reload, fnlist, insizes, outsizes, intypes, outtypes, simnest_name, irpar_fname, dfeed, userdata) // PARSEDOCU_BLOCK
// %PURPOSE: A higher level for defining online replaceable schematics using callback functions
//
// inlist - input list()
// outlist - output list()
// 
// For an example see modules/nested/demo/online_replacement
//    


function [sim, outlist] = exch_helper_thread(sim, inlist)
      // This superblock will run the evaluation of the experiment in a thread.
      // The superblock describes a sub-simulation, whereby only one step is simulated
      // which is enough to call scilab one signle time


  defaultevents = 0;
    
  inputv = inlist(1);

//   [sim] = ld_printf(sim, defaultevents, inputv, "inputv = ", 10);

  //
  // A resource demanding Scilab calculation
  //
  
  [sim, dummyin] = ld_const(sim, defaultevents, 1);

      [sim, compready]  = ld_const(sim, defaultevents, 1);
//      [sim, result]  = ld_const_vec(sim, defaultevents, 123);
      [sim, result] = ld_constvec(sim, defaultevents, vec=1:10)

  // replace schematic RST_ident
  [sim, exchslot] = ld_const(sim, defaultevents, 2);                            
  [sim, out] = ld_nested_exchffile(sim, defaultevents, compresult=compready, slot=exchslot, ... 
                                   fname=irpar_fname, simnest_name);


  // output of schematic
  outlist = list(result);
endfunction




function [sim, outlist, userdata ] = replaceable_cntrl_main(sim, inlist, par)
//    
//    The nested simulation contains two sub-simulations:
//    
//    1) A schematic, which commonly contains nothing and is switched to
//       when the replacement is in progress (which may take some time)
//
//    2) The schematic, which actually contains the algorithm to execute
//
//    Here, the initial simulations are defined, which can then be 
//   replaced during runtime
//    


  ev = 0;

  cntrlN = par(1); // the number of the nested schematics (one of two) "1" means the 
                   // dummy schematic which is activated while the 2nd "2" is exchanged during runtime
  userdata = par(2);
  
  useruserdata = userdata(1);
  define_fn = userdata(2);
  
//  input1 = inlist(1);
  
  
//   // a zero
//   [sim, zero] = ld_const(sim, ev, 0);

 // printf("Defining a replaceable controller\n");

  if (cntrlN == 1) then  // is this the schematic 2) ?
    [sim, outlist, useruserdata] = define_fn( sim, inlist, schematic_type='spare', useruserdata );

  end
  
  if (cntrlN == 2) then  // is this the schematic 2) ?

    [sim, outlist, useruserdata] = define_fn( sim, inlist, schematic_type='default', useruserdata );

//          // Here the  default controller can be defined
//          [sim, output] = ld_gain(sim, ev, input1, 5); // 
//          [sim] = ld_printf(sim, ev, in=zero, str="This is just a dummy simulation, which can be replaced.", insize=1);
  end

  userdata(1) = useruserdata;



//   outlist = list(zero);
endfunction

  [sim,zero] = ld_const(sim, ev, 0);
  [sim,active_sim] = ld_const(sim, ev, 1);

  // get the callback functions
  define_fn = fnlist(1);


  //
  // Here the controller is nested, which can be replaced online
  //

        [sim, outlist_42342, computation_finished, userdata] = ld_simnest2(sim, ev=[ ev ] , ...
                       inlist, ...
                       insizes, outsizes, ...
                       intypes, ...
                       outtypes, ...
                       nested_fn=replaceable_cntrl_main, Nsimulations=2, dfeed, ...
                       asynchron_simsteps=0, switch_signal=active_sim, ...
                       reset_trigger_signal=zero, userdata=list(userdata, define_fn), ...
                       simnest_name );

//        [sim] = ld_printf(sim, ev, in=outlist(1), str="The nested, replaceable sim returns", insize=1);


  //
  // The exchange helper, which consits of a threaded sub-simulation
  // for loading the schematic from files.
  // The replacement is triggered by setting "startcalc" to 1 for one sample
  //

        // input to the calculation
        [sim, input1] = ld_constvec(sim, ev, vec=1:10);

        [sim, zero] = ld_const(sim, ev, 0);
//        [sim, startcalc] = ld_initimpuls(sim, ev);
        startcalc = trigger_reload;


        // a nested simulation that runns asynchronously (in a thread) to the main simulation
        [sim, outlist__, computation_finished] = ld_simnest(sim, ev, ...
                              inlist=list(input1), ...
                              insizes=[10], outsizes=[10], ...
                              intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT], ...
                              fn_list=list(exch_helper_thread), ...
                              dfeed=1, asynchron_simsteps=1, ...
                              switch_signal=zero, reset_trigger_signal=startcalc         );

         output1 = outlist__(1);
         // computation_finished is one, when finished else zero

         replaced = computation_finished; // FIXME and successful
         
         // The output list() of the nested schematic
         outlist = outlist_42342;
         
         
endfunction








function [sim] = ld_global_memory(sim, events, ident_str, datatype, len, initial_data, visibility, useMutex)  // PARSEDOCU_BLOCK
// 
// %PURPOSE: inittialise a persistent globally shared memory
// 
// ident_str (string) - name of the memory
// datatype - ORTD datatype of the memory (for now only ORTD.DATATYPE_FLOAT)
// len (integer) - number of elements
// initial_data - initial data of the memory
// visibility (string) - 'global', ... (more are following)
// useMutex (integer) - 0 or 1. Use a mutex if you access the memory from different threads
// 
// 

  ident_str = ident_str + '.memory';

  if (visibility == 'global') then
  
  else
    error("Visibility has to be one of global, ... (more are following)");
  end

  btype = 15001 + 4   ;	
  ipar = [0, datatype, len, useMutex, 0,0,0,0, 0,0, length(ident_str), ascii(ident_str) ]; 
  
  if datatype == ORTD.DATATYPE_FLOAT then
    rpar = [ initial_data ];
    if (length(initial_data) ~= len) then
      error("length(initial_data) ~= len");
    end
  else
    rpar = [ ];
    error("datatype is not one of ORTD.DATATYPE_FLOAT, ...");
  end

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                   insizes=[], outsizes=[], ...
                   intypes=[], outtypes=[]  );
 
 // ensure the block is included in the simulation even without any I/O ports
 sim = libdyn_include_block(sim, blk);
 
//   [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim] = ld_write_global_memory(sim, events, data, index, ident_str, datatype, ElementsToWrite)   // PARSEDOCU_BLOCK
// 
// %PURPOSE: Write a portion to a persistent globally shared memory
// 
// Initialises a memory structure which can be refered by an
// identifier. Data is available for read and write access
// accross different state machines as well as accross
// different threads.
// 
// Make sure to only use the memory created by this function in 
// lower level simulations such as nested state machines, etc.
// Access from higher level simulations is possible but should
// be avoided, as the memory can not be freed on destruction.
// 
// data *+(ElementsToWrite) - data
// index * - index to store the data. Starts at 1
// ident_str (string) - name of the memory
// datatype - ORTD datatype of the memory (for now only ORTD.DATATYPE_FLOAT)
// ElementsToWrite (integer) - number of elements to write to the memory
// 
// 

  ident_str = ident_str + '.memory';

  btype = 15001 + 5   ;	
  ipar = [0, datatype, ElementsToWrite, 0, 0,0,0,0, 0,0, length(ident_str), ascii(ident_str) ]; 
  rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                   insizes=[ElementsToWrite , 1], outsizes=[], ...
                   intypes=[ datatype, ORTD.DATATYPE_FLOAT  ], outtypes=[]  );
 
 // ensure the block is included in the simulation even without any I/O ports
 [sim,blk] = libdyn_conn_equation(sim, blk, list(data, index) );
 
 
 
//   [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

function [sim, data] = ld_read_global_memory(sim, events, index, ident_str, datatype, ElementsToRead)   // PARSEDOCU_BLOCK
// 
// %PURPOSE: Read a portion from a persistent globally shared memory
// 
// data *+(ElementsToRead) - data
// index * - index to store the data. Starts at 1
// ident_str (string) - name of the memory
// datatype - ORTD datatype of the memory (for now only ORTD.DATATYPE_FLOAT)
// ElementsToRead (integer) - number of elements to read from the memory
// 

  ident_str = ident_str + '.memory';

  btype = 15001 + 6;
  ipar = [0, datatype, ElementsToRead, 0, 0,0,0,0, 0,0, length(ident_str), ascii(ident_str) ]; 
  rpar = [];

  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                   insizes=[1], outsizes=[ElementsToRead], ...
                   intypes=[ ORTD.DATATYPE_FLOAT ], outtypes=[datatype*[1]]  );
 
 // ensure the block is included in the simulation even without any I/O ports
 [sim,blk] = libdyn_conn_equation(sim, blk, list(index) );
 
   [sim,data] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction





//
//    Copyright (C) 2010, 2011  Christian Klauer
//
//    This file is part of OpenRTDynamics, the Real Time Dynamic Toolbox
//
//    OpenRTDynamics is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OpenRTDynamics is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with OpenRTDynamics.  If not, see <http://www.gnu.org/licenses/>.
//





// Interfacing functions are placed in this place



function [sim,out] = ld_parameter(sim, events, str, initial_param) // PARSEDOCU_BLOCK
  // Creates a new parameter block that is remotely controlable via TCP
  // It requires the set-up of a libdyn master
  // 
  // str - is a string of the parameter name
  // initial_param - is a vector of the initial parameter set
  // out - is a vectorial signal of size length(initial_param)
  // 
  Nin = 0;
  Nout = 1;

  btype = 14001;
  str = ascii(str);
  nparam = length(initial_param);
  
  ipar = [0, nparam, length(str), str(:)'];
  rpar = [initial_param(:)];
  
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[], outsizes=[nparam], ...
                       intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );


//   [sim,blk] = libdyn_conn_equation(sim, blk, inlist);
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim] = ld_stream(sim, events, in, str, insize, autoflushInterval, autoflushTimeout, bufferlen) // PARSEDOCU_BLOCK
  // Creates a new stream block that is remotely controlable via TCP
  // It requires the set-up of a libdyn master
  // 
  // str - is a string of the stream name
  // insize is the vector length of the input port
  // [autoflushInterval] how often (in samples) to flush the send buffers
  // [autoflushTimeout] how often to flush the send buffers (time difference) (not implemented)
  // [bufferlen] number of samples within the ringbuffer
  //

  // set some defeult values for optional variables if not present
  if (exists('bufferlen') == 0)
    bufferlen = 1000; // how many vectors should be stored in the ringbuffer
  end

  if (exists('autoflushInterval') == 0)
    autoflushInterval = 1;
  end

  if (exists('autoflushTimeout') == 0)
    autoflushTimeout = 0.1;
  end

  datatype = -1; // FIXME set to FLOAT


  btype = 14001 + 1;
  str = ascii(str);

  
  ipar = [0, insize, datatype, bufferlen, autoflushInterval, length(str), str(:)'];
  rpar = [autoflushTimeout ];
  
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                       insizes=[insize], outsizes=[], ...
                       intypes=[ORTD.DATATYPE_FLOAT], outtypes=[]  );

  // connect input port
  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
endfunction

// Interfacing functions are placed in this place





// An example could be the following scilab function. PARSEDOCU_BLOCK is a hint for the documentation generator to generate documentation for this block
function [sim, outlist] = ld_scicosblock(sim, events, inlist, cosblk) // PARSEDOCU_BLOCK
// 
// %PURPOSE: include a scicos block / schematic (EXPERIMENTAL)
//
// inlist  - list() of input ports forwarded to the Scicosblock
// outlist  - list() of output ports forwarded from the Scicosblock
// 
// cosblk - structure as loaded by ortd_getcosblk containg the block parameters 
//

  

  model = cosblk;

//   identstr = model.blockname;
  identstr = model.sim(1); // name of C-comp fn

  printf("Including Scicos Block %s\n", identstr);
  printf("Make sure that the C function %s() is included in the binary used for execution\n", identstr);

  
//    if ( length(cosblk.in) ~= 1 ) then
//      error("Only one input port to the Scicos-Block allowed at the moment");
//    end
//  
//    if ( length(cosblk.out) ~= 1 ) then
//      error("Only one output port to the Scicos-Block allowed at the moment");
//    end
 
 
   Nin = length(cosblk.in);
   Nout = length(cosblk.out);

  
//           if ( irpar_get_ivec(&insizes, ipar, rpar, 10) < 0 ) error = 1 ;
//         if ( irpar_get_ivec(&outsizes, ipar, rpar, 11) < 0 ) error = 1 ;
//         if ( irpar_get_ivec(&intypes, ipar, rpar, 12) < 0 ) error = 1 ;
//         if ( irpar_get_ivec(&outtypes, ipar, rpar, 13) < 0 ) error = 1 ;
//         if ( irpar_get_ivec(&param, ipar, rpar, 18) <  0 ) error = 1 ;
//         
//         int dfeed = param.v[1];
// 
//     
//     if ( irpar_get_ivec(&identstr__, ipar, rpar, 20) < 0 ) error = 1 ;
//     if ( irpar_get_ivec(&block_ipar, ipar, rpar, 21) < 0 ) error = 1 ;
//     if ( irpar_get_rvec(&block_rpar, ipar, rpar, 22) < 0 ) error = 1 ;
//     if ( irpar_get_rvec(&dstate, ipar, rpar, 23) < 0 ) error = 1 ;
//     
  

  dfeed = 1;
  parameters = [dfeed];
  
  parlist = new_irparam_set();

  parlist = new_irparam_elemet_ivec(parlist, cosblk.in, 10);
  parlist = new_irparam_elemet_ivec(parlist, cosblk.out, 11);
  parlist = new_irparam_elemet_ivec(parlist, ORTD.DATATYPE_FLOAT*ones(cosblk.in) , 12); // only float is supported by now
  parlist = new_irparam_elemet_ivec(parlist, ORTD.DATATYPE_FLOAT*ones(cosblk.out) , 13); // only float is supported by now
  parlist = new_irparam_elemet_ivec(parlist, parameters, 18);


  parlist = new_irparam_elemet_ivec(parlist, ascii(identstr), 20);
  parlist = new_irparam_elemet_ivec(parlist, model.ipar, 21);
  parlist = new_irparam_elemet_rvec(parlist, model.rpar, 22);
  parlist = new_irparam_elemet_rvec(parlist, model.dstate, 23);

  blockparam = combine_irparam(parlist);



 btype = 15200 + 0;; // the same id you are giving via the "libdyn_compfnlist_add" C-function
//  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ 0, Nin, Nout, length(identstr), ascii(identstr)  ], rpar=[  ], ...
//                   insizes=[Nin], outsizes=[Nout], ...
//                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );



 [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ blockparam.ipar(:)'  ], rpar=[ blockparam.rpar(:)' ], ...
                  insizes=[  cosblk.in(:)' ], outsizes=[ cosblk.out(:)' ], ...
                  intypes=[ones(1,Nin)*ORTD.DATATYPE_FLOAT], outtypes=[ones(1,Nout)*ORTD.DATATYPE_FLOAT]  );


                  
   [sim,blk] = libdyn_conn_equation(sim, blk, inlist );
 
   outlist = list();
  for i = 0:(Nout-1)
    [sim,out] = libdyn_new_oport_hint(sim, blk, i);   // ith port
    outlist(i+1) = out;
  end

 
 //[sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction




function cosblk=ortd_getcosblk2(blockname, flag, cachefile)  // PARSEDOCU_BLOCK
  //
  // %PURPOSE: Extract information from Scicos block interfacing function macros (*.sci) files
  //
  // pathtoscifile - The interfacing function macro (a *.sci file)
  // flag - one of 'defaults', 'rundialog'
  //


  //exec(blockname + '_c.sci');
//   exec(pathtoscifile);

//  Scicos
//
//  Copyright (C) INRIA - METALAU Project <scicos@inria.fr>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// See the file ../license.txt
//

function model=scicos_model(v1,v2,v3,v4,v5,v6,v7,v8,v9,v10,v11,v12,v13,v14,v15,v16,...
                            v17,v18,v19,v20,v21,v22,v23,v24,v25,v26)

  //initialisation de model mlist
  if exists('sim','local')==0 then sim='',end
  if exists('in','local')==0 then in=[],end
  if exists('in2','local')==0 then in2=[],end
  if exists('intyp','local')==0 then intyp=1,end
  if exists('out','local')==0 then out=[],end
  if exists('out2','local')==0 then out2=[],end
  if exists('outtyp','local')==0 then outtyp=1,end
  if exists('evtin','local')==0 then evtin=[],end
  if exists('evtout','local')==0 then evtout=[],end
  if exists('state','local')==0 then state=[],end
  if exists('dstate','local')==0 then dstate=[],end
  if exists('odstate','local')==0 then odstate=list(),end
  if exists('opar','local')==0 then opar=list(),end
  if exists('rpar','local')==0 then rpar=[],end
  if exists('ipar','local')==0 then ipar=[],end
  if exists('blocktype','local')==0 then blocktype='c',end
  if exists('firing','local')==0 then firing=[],end
  if exists('dep_ut','local')==0 then dep_ut=[%f %f],end
  if exists('label','local')==0 then label='',end
  if exists('nzcross','local')==0 then nzcross=0,end
  if exists('nmode','local')==0 then nmode=0,end
  if exists('equations','local')==0 then equations=list(),end

//  model=mlist(['model','sim','in','in2','intyp','out','out2','outtyp',...
//               'evtin','evtout','state','dstate','odstate','rpar','ipar','opar',...
//               'blocktype','firing','dep_ut','label','nzcross',..
//               'nmode','equations'],..
//               sim,in,in2,intyp,out,out2,outtyp,evtin,evtout,state,dstate,odstate,..
//               rpar,ipar,opar,blocktype,firing,dep_ut,label,nzcross,nmode,equations)

			model.sim=sim,..
			model.in=in,..
			model.in2=in2,..
			model.intyp=intyp,..
			model.out=out,..
			model.out2=out2,..
			model.outtyp=outtyp,..
//			model.evtin=clkinput,..
//			model.evtout=clkoutput,..
			model.firing=firing,..
			model.state=state,..
			model.dstate=dstate,..
			model.odstate=odstate,..
			model.rpar=rpar,..
			model.ipar=ipar,..
			model.opar=opar,..
			model.blocktype='c',..
			model.dep_ut=dep_ut,..
			model.nzcross=nzcross,..
			model.nmode=nmode  
endfunction


  function [model,graphics,ok]=check_io(model,graphics,in,out,clkin,clkout,in_implicit,out_implicit)
    // no actual check, just a copy into the model structure
    ok=%t


    model.evtin=clkin
    model.evtout=clkout
    model.in = in;
    model.out = out;
    model.evkin = clkin;
    model.evout = clkout;
  endfunction


  function x=standard_define( a, b, c, d )
    x.a =  a;
    x.b = b; // model
    x.c = c(:); // exprs
    x.d = d;

    x.a =  a;
    x.model = b; // model
    x.graphics.exprs = c(:); // exprs
    x.d = d;
  endfunction
  
  if (flag == 'usecachefile') then
      [lhs,rhs]=argn(0);
      if rhs >= 3 then
        try
          load(cachefile);  // ideally results in a new variable X, which is a structure
          printf("Using cachefile %s\n", cachefile);
          
          cosblk = X.model;
          cosblk.timestamp = getdate();
          cosblk.blockname = blockname;

        catch
          1;
        end
        1;
      else
        error("No cachefile was specified");
      end

      
  end
  
  if (flag == 'rundialog') then
  
    // overwrite a potenially already available X, which does not belong this code.
    X=0; // make typeof X to be constant
  
    // check for cached cosblk
      [lhs,rhs]=argn(0);
      if rhs >= 3 then
        try
          load(cachefile);  // ideally results in a new variable X, which is a structure
          printf("Using cachefile %s\n", cachefile);
        catch
          1;
        end
        1;
      end
      
    // if not available, call standard_define
    if (typeof(X) == 'constant') then // no new X was loaded, call macro for default values
      printf("loading block''s standard parameters\n");
    
      definecommand = "" + blockname + "(job=''define'',arg1=0,arg2=0);";
      X = eval(definecommand);
//       cosblk = X.b;
    end

    

//     arg1.graphics.exprs = X.graphics.exprs;
    arg1.graphics = X.graphics;
    arg1.model = X.model;
    definecommand = "" + blockname + "(job=''set'',arg1,arg2=0);";
    X = eval(definecommand);

    
    cosblk = X.model;
    cosblk.timestamp = getdate();
    cosblk.blockname = blockname;
    
    // check for cached cosblk
    [lhs,rhs]=argn(0) 
    if rhs >= 3 then
      printf("Saving cachefile %s\n", cachefile);
      save(cachefile, X);
    end
      

  end
//  clear X;

endfunction




function cosblk=ortd_getcosblk(blockname, pathtoscifile)  // PARSEDOCU_BLOCK
  //
  // %PURPOSE: Extract information from Scicos block interfacing function macros (*.sci) files
  //
  // pathtoscifile - The interfacing function macro (a *.sci file)
  //

  
  // check if pathtoscifile is a function

  //exec(blockname + '_c.sci');
//   if (typeof(pathtoscifile) == 'string') then
   if pathtoscifile ~= "" then
     exec(pathtoscifile);
   end
//   end

  function model = scicos_model(sim,..
			in,..
			in2,..
			intyp,..
			out,..
			out2,..
			outtyp,..
			evtin,..
			evtout,..
			firing,..
			state,..
			dstate,..
			odstate,..
			rpar,..
			ipar,..
			opar,..
			blocktype,..
			dep_ut,..
			nzcross,..
			nmode )
			
			model.sim=sim,..
			model.in=in,..
			model.in2=in2,..
			model.intyp=intyp,..
			model.out=out,..
			model.out2=out2,..
			model.outtyp=outtyp,..
			model.evtin=clkinput,..
			model.evtout=clkoutput,..
			model.firing=firing,..
			model.state=x,..
			model.dstate=Z,..
			model.odstate=odstate,..
			model.rpar=rpar,..
			model.ipar=ipar,..
			model.opar=opar,..
			model.blocktype='c',..
			model.dep_ut=dep_ut,..
			model.nzcross=nzcross,..
			model.nmode=nmode           
			
  endfunction

  function x=standard_define( a, b, c, d )
    x.a =  a;
    x.b = b;
    x.c = c;
    x.d = d;
  endfunction

  //definecommand = "[x,y,typ]=" + blockname + "_c(job=''define'',arg1=0,arg2=0);";
  definecommand = "" + blockname + "_c(job=''define'',arg1=0,arg2=0);";
  X = eval(definecommand);
  cosblk = X.b;
  cosblk.timestamp = getdate();
  cosblk.blockname = blockname;
  clear X;

endfunction

// function cosblk=ortd_getcosblk2(blockname, pathtoscifile)  // PARSEDOCU_BLOCK
//   //
//   // %PURPOSE: Extract information from Scicos block interfacing function macros (*.sci) files
//   //
//   // pathtoscifile - The interfacing function macro (a *.sci file)
//   //
// 
// 
//   //exec(blockname + '_c.sci');
//   exec(pathtoscifile);
// 
//   function model = scicos_model(sim,..
// 			in,..
// 			in2,..
// 			intyp,..
// 			out,..
// 			out2,..
// 			outtyp,..
// 			evtin,..
// 			evtout,..
// 			firing,..
// 			state,..
// 			dstate,..
// 			odstate,..
// 			rpar,..
// 			ipar,..
// 			opar,..
// 			blocktype,..
// 			dep_ut,..
// 			nzcross,..
// 			nmode )
// 			
// 			model.sim=sim,..
// 			model.in=in,..
// 			model.in2=in2,..
// 			model.intyp=intyp,..
// 			model.out=out,..
// 			model.out2=out2,..
// 			model.outtyp=outtyp,..
// 			model.evtin=clkinput,..
// 			model.evtout=clkoutput,..
// 			model.firing=firing,..
// 			model.state=x,..
// 			model.dstate=Z,..
// 			model.odstate=odstate,..
// 			model.rpar=rpar,..
// 			model.ipar=ipar,..
// 			model.opar=opar,..
// 			model.blocktype='c',..
// 			model.dep_ut=dep_ut,..
// 			model.nzcross=nzcross,..
// 			model.nmode=nmode           
// 			
//   endfunction
// 
//   function x=standard_define( a, b, c, d )
//     x.a =  a;
//     x.b = b;
//     x.c = c;
//     x.d = d;
//   endfunction
// 
//   definecommand = "" + blockname + "(job=''define'',arg1=0,arg2=0);";
//   X = eval(definecommand);
//   cosblk = X.b;
//   cosblk.timestamp = getdate();
//   cosblk.blockname = blockname;
//   clear X;
// 
// endfunction
// 
// model=ortd_getcosblk(blockname="SuperBlock");
// save(blockname+"_scicosblockcfg.dat", model);




// Interfacing functions are placed into this place



function [sim, out] = ld_scilab(sim, events, in, invecsize, outvecsize, init_cmd, calc_cmd, destruct_cmd, scilab_path) // PARSEDOCU_BLOCK
// %PURPOSE: Block for interfacing scilab
//
// in *+(invecsize) - input        scilab_interf.invec%d = [ .... ];
// out *+(outvecsize) - output        scilab_interf.outvec%d = [ .... ];
// scilab_path - Path to scilab5 executable
// 
// out = calc_cmd        scilab_interf.outvec%d = calc_cmd(scilab_interf.invec%d);
//
// NOTE: The 64-Bit verion of scilab is not working for some reason,
//       but also on 64-Bit systems you can use the 32-Bit version
//       if you add "linux32" in front of the command that starts
//       the controller process, as shown in demo/run.sh
// 


  invecno=1; outvecno=1;

  init_cmd = ascii(init_cmd);
  calc_cmd = ascii(calc_cmd);
  destruct_cmd = ascii(destruct_cmd);
  scilab_path = ascii(scilab_path);
  
  btype = 22000;
  ipar = [invecsize; outvecsize; invecno; outvecno; 0; 0; 0; 0; 0; 0; 0; length(init_cmd); length(calc_cmd); ...
                   length(destruct_cmd); length(scilab_path); (init_cmd)';...
                   (calc_cmd)'; (destruct_cmd)'; (scilab_path)'; 0; ]; rpar = [];
  
//   pause;
  
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar, rpar, ...
                   insizes=[invecsize], outsizes=[outvecsize], ...
                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

  // libdyn_conn_equation connects multiple input signals to blocks
  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


// Computaional function example:
// 
//function [block]=sample_comp_fn( block, flag )
//  select flag
//    case 0 // update
//      printf("update states\n");
//
//    case 1 // output
//      printf("update outputs\n");
//      outvec = [1:6]';
//
//      block.outptr(1) = outvec;
//
//    case 4 // init
//      printf("init\n");
//
//    case 5 // terminate
//      printf("terminate\n");
//
//    case 10 // configure
//      printf("configure\n");
//      block.invecsize = 5;
//      block.outvecsize = 6;
//
//  end
//endfunction

function [sim, out] = ld_scilab2(sim, events, in, comp_fn, include_scilab_fns, scilab_path) // PARSEDOCU_BLOCK
// %PURPOSE: Block for calling a computational function written in scilab
//
// A nicer interface to scilab. (See modules/scilab/demo/scilab_demo.sce for an example)
//
// in *+(invecsize) - input
// out *+(outvecsize) - output
// 
// out = calc_cmd
//
//
// comp_fn - scilab function that is executed online. Prototype:
//           function [block]=sample_comp_fn( block, flag )
//
//           flags are: 1 calc outputs, 4 initialise, 5 terminate, 10 configure I/O
//
//           For further details have a look at the example.
//
// include_scilab_fns - unused for now. Set to list()
// scilab_path - Path to scilab5 executable
//
// NOTE: For now the flag "update states" is not called; Also a "reset states" flag is required.
//
// NOTE: The 64-Bit verion of scilab is not working for some reason,
//       but also on 64-Bit systems you can use the 32-Bit version
//       if you add "linux32" in front of the command that starts
//       the controller process, as shown in demo/run.sh
//

  function [result] = check_structure( str )
    // give something like str="a.b" to see wheter this would lead to an error

    result = %T;
    err = execstr(str, 'errcatch');
    if (err ~= 0) then
      result = %F;
    end
  endfunction

  block.dummy = 0;

  // Get the number of I/O
  block = comp_fn(block, 10);

  if check_structure("block.invecsize") == %F then
    error("comp_fn did not return block.invecsize\n");
  end

  if check_structure("block.outvecsize") == %F then
    error("comp_fn did not return block.outvecsize\n");
  end

  invecsize = block.invecsize;
  outvecsize = block.outvecsize;

  // make source code from the computational function
  function_string = fun2string(comp_fn);
  function_string(1) = strsubst(function_string(1), "ans", "comp_fn"); // rename the function

  init_cmd="";

  LF = char(10);

  for i=1:length(length(function_string))
    init_cmd = init_cmd + function_string(i) + LF; // concate lines and add a line feed character
  end
//  init_cmd = init_cmd + "comp_fn = ans;" + LF;

  // Prepare initialisation string, which contains the comp_fn as source
  init_str = "" + LF + ...
             "block.invecsize=" + string(invecsize) + ";" + LF + ...
             "block.outvecsize=" + string(outvecsize) + ";" + LF + ...
             "block=comp_fn(block, 4);" + LF + ...
             "printf(''scilab computational function initialised\n''); " + LF + ...
             "block.inptr = list(1);" + LF + ...
             "block.outptr = list(1);" + LF;

  init_cmd = init_cmd + init_str;

  // Calc command
  calc_cmd = "block.inptr(1) = scilab_interf.invec1;" + ...
             "block=comp_fn(block, 1); " + ...
             "scilab_interf.outvec1 = block.outptr(1); " + ... // all in one line
             "" + LF;

  // command for destruction
  destruct_cmd = "" + LF + ...
             "block=comp_fn(block, 5);" + LF + ...
             "printf(''scilab computational function destructed\n''); " + LF + ...
             "clear block;" + LF;

//  fd = mopen('init_cmd.txt','wt');  mputl(init_cmd,fd);  mclose(fd);
//  fd = mopen('calc_cmd.txt','wt');  mputl(calc_cmd,fd);  mclose(fd);
//  fd = mopen('destruct_cmd.txt','wt');  mputl(destruct_cmd,fd);  mclose(fd);

//pause;

  // auf bereits bekannte funktion zurückführen
  [sim, out] = ld_scilab(sim, events, in, invecsize, outvecsize,  ...
    init_cmd, calc_cmd, destruct_cmd, scilab_path);

endfunction






// Interfacing functions are placed in this place


function [sim,out] = ld_serial_interface(sim, events, in, devname, send_cmd, rcv_cmd, inSize, outSize, BaudRate)
// 
// 
// 
// 
// 
// 
// 

  btype = 16001;

  devname_as = ascii(devname);
  send_cmd_as = ascii(send_cmd);
  rcv_cmd_as = ascii(rcv_cmd);
  
  ipar = [0, inSize, outSize, BaudRate, 0, 0, 0, length(devname_as), length(send_cmd_as), length(rcv_cmd_as),  devname_as(:)', send_cmd_as(:)', rcv_cmd_as(:)', ];
  rpar = [ ];
  
  [sim,blk] = libdyn_new_blk_generic(sim, events, btype, ipar, rpar);


  [sim,blk] = libdyn_conn_equation(sim, blk, list(in));
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction



// Interfacing functions are placed in this place


// ADJUST HERE: Add your interfacing functions here



function [sim, out] = ld_synctimer(sim, events, in) // PARSEDOCU_BLOCK
// %PURPOSE: Timer for synchronisation of a async simulation 
//
// To beused within an async nested schematic for introducing variable sample times (EXPERIMENTAL)
// 
// For an example see modules/synchronisation/demo/timed_thread.sce
// 
// 


 btype = 15100 + 0; //
 [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ 0  ], rpar=[ ], ...
                  insizes=[1], outsizes=[1], ...
                  intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );

 [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
 [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


function [sim, out] = ld_clock(sim, events) // PARSEDOCU_BLOCK
//
// %PURPOSE: get current system time in [s]
//
// out * - time
// 
// 


  btype = 15100 + 1;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ 0 ], rpar=[  ], ...
                   insizes=[], outsizes=[1], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );


  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction


// Interfacing functions are placed in this place


// ADJUST HERE: Add your interfacing functions here

// An example could be the following scilab function. PARSEDOCU_BLOCK is a hint for the documentation generator to generate documentation for this block
//function [sim, out] = ld_abs(sim, events, in) // PARSEDOCU_BLOCK
//// ADD SOME DOCUMENTATION HERE
//// abs - block
////
//// in * - input
//// out * - output
//// 
//// out = abs(in)
//// 
//
//  btype = 60001 + 7; // the same id you are giving via the "libdyn_compfnlist_add" C-function
//  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ <ADD INTEGERS HERE>  ], rpar=[ <ADD FLOATING POITN VALUES HERE> ], ...
//                   insizes=[1], outsizes=[1], ...
//                   intypes=[ORTD.DATATYPE_FLOAT], outtypes=[ORTD.DATATYPE_FLOAT]  );
//
//  [sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
//  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
//endfunction

// if you like to put strings as arguments for the block you could used ipar=[ length(str), ascii(str) ] for example and use "irpar_getstr" within C-Code for decoding (see irpar.h and irpar.c)


// Interfacing functions are placed in this place

function [sim, out] = ld_udp_main_receiver(sim, events, udpport, identstr, socket_fname, vecsize) // PARSEDOCU_BLOCK
// udp main receiver - block
//
// 

  datatype = ORTD.DATATYPE_FLOAT;

  btype = 39001;
  [sim,blk] = libdyn_new_block(sim, events, btype, ipar=[ udpport, vecsize, datatype, length(socket_fname), ascii(socket_fname), length(identstr), ascii(identstr) ], rpar=[  ], ...
                   insizes=[], outsizes=[vecsize], ...
                   intypes=[], outtypes=[ORTD.DATATYPE_FLOAT]  );

  //[sim,blk] = libdyn_conn_equation(sim, blk, list(in) );
  [sim,out] = libdyn_new_oport_hint(sim, blk, 0);   // 0th port
endfunction

